#!/usr/bin/python
# encoding: utf-8

# Copyright 2016-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=broad-except,global-variable-undefined

'''
pb_addNewServer -- adds a new server to a data center

pb_addNewServer is a tool to add a new server/storage composite to an
existing data center. The server may be set up by a disk or cdrom image.


@author:     Jürgen Buchhammer

@copyright:  2016 IONOS GmbH. All rights reserved.

@license:    Apache License 2.0

@deffield    updated: Updated
'''

import sys
import os
import traceback

from time import sleep
from datetime import datetime

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from base64 import b64encode, b64decode

from ionosenterprise.client import IonosEnterpriseService, Server, Volume, NIC


__all__ = []
__version__ = 0.1
__date__ = '2016-02-16'
__updated__ = '2016-02-16'


class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg

    def __str__(self):
        return self.msg

    def __unicode__(self):
        return self.msg
# end class CLIError


def getLogin(filename, user, passwd):
    '''
    write user/passwd to login file or get them from file.
    This method is not Py3 safe (byte vs. str)
    '''
    if filename is None:
        return (user, passwd)
    if os.path.exists(filename):
        print("Using file {} for Login".format(filename))
        with open(filename, "r") as loginfile:
            encoded_cred = loginfile.read()
            decoded_cred = b64decode(encoded_cred)
            login = decoded_cred.split(':', 1)
            return (login[0], login[1])
    else:
        if user is None or passwd is None:
            raise ValueError("user and password must not be None")
        print("Writing file {} for Login".format(filename))
        with open(filename, "w") as loginfile:
            encoded_cred = b64encode(user+":"+passwd)
            loginfile.write(encoded_cred)
        return (user, passwd)
# end getLogin()


def wait_for_request(pbclient, request_id,
                     timeout=0, initial_wait=5, scaleup=10):
    '''
    Waits for a request to finish until timeout.
    timeout==0 is interpreted as infinite wait time.
    Returns a tuple (return code, request status, message) where return code
    0  : request successful
    1  : request failed
    -1 : timeout exceeded
    The wait_period is increased every scaleup steps to adjust for long
    running requests.
    '''
    total_wait = 0
    wait_period = initial_wait
    next_scaleup = scaleup * wait_period
    wait = True
    while wait:
        request_status = pbclient.get_request(request_id, status=True)
        state = request_status['metadata']['status']
        if state == "DONE":
            return(0, state, request_status['metadata']['message'])
        if state == 'FAILED':
            return(1, state, request_status['metadata']['message'])
        if verbose > 0:
            print("Request '{}' is in state '{}'. Sleeping for {} seconds..."
                  .format(request_id, state, wait_period))
        sleep(wait_period)
        total_wait += wait_period
        if timeout != 0 and total_wait > timeout:
            wait = False
        next_scaleup -= wait_period
        if next_scaleup == 0:
            wait_period += initial_wait
            next_scaleup = scaleup * wait_period
            if verbose > 0:
                print("scaling up wait_period to {}, next change in {} seconds"
                      .format(wait_period, next_scaleup))
    # end while(wait)
    return(-1, state, "request not finished before timeout")
# end wait_for_request()


def main(argv=None):
    '''Parse command line options and create a server/volume composite.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version,
                                                     program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by J. Buchhammer on %s.
  Copyright 2016 IONOS. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license,
                                formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-u', '--user', dest='user', help='the login name')
        parser.add_argument('-p', '--password', dest='password',
                            help='the login password')
        parser.add_argument('-L', '--Login', dest='loginfile', default=None,
                            help='the login file to use')
        parser.add_argument('-d', '--datacenterid', dest='datacenterid',
                            required=True, default=None,
                            help='datacenter of the new server')
        parser.add_argument('-l', '--lanid', dest='lanid', required=True,
                            default=None, help='LAN of the new server')
        parser.add_argument('-n', '--name', dest='servername',
                            default="SRV_"+datetime.now().isoformat(),
                            help='name of the new server')
        parser.add_argument('-c', '--cores', dest='cores', type=int,
                            default=2, help='CPU cores')
        parser.add_argument('-r', '--ram', dest='ram', type=int, default=4,
                            help='RAM in GB')
        parser.add_argument('-s', '--storage', dest='storage', type=int,
                            default=4, help='storage in GB')
        parser.add_argument('-b', '--boot', dest='bootdevice', default="HDD",
                            help='boot device')
        parser.add_argument('-i', '--imageid', dest='imageid', default=None,
                            help='installation image')
        parser.add_argument('-P', '--imagepassword', dest='imgpassword',
                            default=None, help='the image password')
        parser.add_argument('-v', '--verbose', dest="verbose", action="count",
                            help="set verbosity level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version',
                            version=program_version_message)

        # Process arguments
        args = parser.parse_args()
        global verbose
        verbose = args.verbose
        dc_id = args.datacenterid
        lan_id = args.lanid
        servername = args.servername

        if verbose > 0:
            print("Verbose mode on")
            print("start {} with args {}".format(program_name, str(args)))

        # Test images (location de/fra)
        # CDROM: 7fc885b3-c9a6-11e5-aa10-52540005ab80   # debian-8.3.0-amd64-netinst.iso
        # HDD:   28007a6d-c88a-11e5-aa10-52540005ab80   # CentOS-7-server-2016-02-01
        hdimage = args.imageid
        cdimage = None
        if args.bootdevice == "CDROM":
            hdimage = None
            cdimage = args.imageid
        print("using boot device {} with image {}"
              .format(args.bootdevice, args.imageid))

        (user, password) = getLogin(args.loginfile, args.user, args.password)
        if user is None or password is None:
            raise ValueError("user or password resolved to None")
        pbclient = IonosEnterpriseService(user, password)

        first_nic = NIC(name="local", ips=[], dhcp=True, lan=lan_id)
        volume = Volume(name=servername+"-Disk", size=args.storage,
                        image=hdimage, image_password=args.imgpassword)
        server = Server(name=servername, cores=args.cores, ram=args.ram*1024,
                        create_volumes=[volume], nics=[first_nic],
                        boot_cdrom=cdimage)
        print("creating server..")
        if verbose > 0:
            print("SERVER: {}".format(str(server)))
        response = pbclient.create_server(dc_id, server)
        print("wait for provisioning..")
        wait_for_request(pbclient, response["requestId"])
        server_id = response['id']
        print("Server provisioned with ID {}".format(server_id))
        nics = pbclient.list_nics(dc_id, server_id, 1)
        # server should have exactly one nic, but we only test empty nic list
        if not nics['items']:
            raise CLIError("No NICs found for newly created server {}"
                           .format(server_id))
        nic0 = nics['items'][0]
        if verbose > 0:
            print("NIC0: {}".format(str(nic0)))
        (nic_id, nic_mac) = (nic0['id'], nic0['properties']['mac'])
        print("NIC of new Server has ID {} and MAC {}".format(nic_id, nic_mac))
        print("{} finished w/o errors".format(program_name))
        return 0

    except KeyboardInterrupt:
        # handle keyboard interrupt #
        return 0
    except Exception:
        traceback.print_exc()
        sys.stderr.write("\n" + program_name + ":  for help use --help\n")
        return 2
# end main()


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=reimported,wrong-import-position

"""Create volume
"""

from ionosenterprise.client import IonosEnterpriseService, Volume

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

i = Volume(
    name='Explicitly created volume',
    size=56,
    image='<IMAGE/SNAPSHOT-ID>',
    bus='VIRTIO')

response = client.create_volume(
    datacenter_id=datacenter_id, volume=i)

"""Create snapshot
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id = '700e1cab-99b2-4c30-ba8c-1d273ddba025'

client = IonosEnterpriseService(
    username='username', password='password')

volume = client.create_snapshot(
    datacenter_id=datacenter_id,
    volume_id=volume_id,
    name='<URLENCODED_SNAPSHOT_NAME>',
    description='<URLENCODED_SNAPSHOT_DESCRIPTION>')

"""Restore Snapshot
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id = '700e1cab-99b2-4c30-ba8c-1d273ddba025'
snapshot_id = '7df81087-5835-41c6-a10b-3e098593bba4'


client = IonosEnterpriseService(
    username='username', password='password')

response = client.restore_snapshot(
    datacenter_id=datacenter_id,
    volume_id=volume_id,
    snapshot_id=snapshot_id)

"""Update Volume
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id = '700e1cab-99b2-4c30-ba8c-1d273ddba025'

client = IonosEnterpriseService(
    username='username', password='password')

volume = client.update_volume(
    datacenter_id=datacenter_id,
    volume_id=volume_id,
    size=100,
    name='Resized storage to 100 GB')



#!/usr/bin/python
# encoding: utf-8

# Copyright 2016-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=broad-except,global-variable-undefined

'''
pb_controlServerState -- control a server's state (start/stop)

pb_controlServerState is a tool to control the state of a server/VM
by stop/start actions


@author:     Jürgen Buchhammer

@copyright:  2016 IONOS. All rights reserved.

@license:    Apache License 2.0

@deffield    updated: Updated
'''

import sys
import os
import traceback
import time

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from base64 import b64encode, b64decode
from subprocess import call

from ionosenterprise.client import IonosEnterpriseService


__all__ = []
__version__ = 0.1
__date__ = '2016-02-25'
__updated__ = '2016-02-25'


class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg

    def __str__(self):
        return self.msg

    def __unicode__(self):
        return self.msg
# end class CLIError


def getLogin(filename, user, passwd):
    '''
    write user/passwd to login file or get them from file.
    This method is not Py3 safe (byte vs. str)
    '''
    if filename is None:
        return (user, passwd)
    if os.path.exists(filename):
        print("Using file {} for Login".format(filename))
        with open(filename, "r") as loginfile:
            encoded_cred = loginfile.read()
#             print("encoded: {}".format(encoded_cred))
            decoded_cred = b64decode(encoded_cred)
            login = decoded_cred.split(':', 1)
            return (login[0], login[1])
    else:
        if user is None or passwd is None:
            raise ValueError("user and password must not be None")
        print("Writing file {} for Login".format(filename))
        with open(filename, "w") as loginfile:
            encoded_cred = b64encode(user+":"+passwd)
#             print("encoded: {}".format(encoded_cred))
            loginfile.write(encoded_cred)
        return (user, passwd)
# end getLogin()


def getServerInfo(pbclient=None, dc_id=None):
    ''' gets info of servers of a data center'''
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    if dc_id is None:
        raise ValueError("argument 'dc_id' must not be None")
    # list of all found server's info
    server_info = []
    # depth 1 is enough for props/meta
    servers = pbclient.list_servers(dc_id, 1)
    for server in servers['items']:
        props = server['properties']
        info = dict(id=server['id'], name=props['name'],
                    state=server['metadata']['state'],
                    vmstate=props['vmState'])
        server_info.append(info)
    # end for(servers)
    return server_info
# end getServerInfo()


def select_where(info=None, select=None, **where):
    if info is None:
        raise ValueError("argument 'info' must not be None")
    if not info:
        return []
    if select is None:
        select = info[0].keys()
    server_info = []
    for old_si in info:
        w_matches = all(old_si[wk] == wv for (wk, wv) in where.items())
        new_si = {k: v for (k, v) in old_si.items() if k in select and w_matches}
        if new_si:
            server_info.append(new_si)
    # end for(info)
    return server_info
# end select_where()


def getServerStates(pbclient=None, dc_id=None, serverid=None, servername=None):
    ''' gets states of a server'''
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    if dc_id is None:
        raise ValueError("argument 'dc_id' must not be None")
    server = None
    if serverid is None:
        if servername is None:
            raise ValueError("one of 'serverid' or 'servername' must be specified")
        # so, arg.servername is set (to whatever)
        server_info = select_where(getServerInfo(pbclient, dc_id),
                                   ['id', 'name', 'state', 'vmstate'],
                                   name=servername)
        if len(server_info) > 1:
            raise NameError("ambiguous server name '{}'".format(servername))
        if len(server_info) == 1:
            server = server_info[0]
    else:
        # get by ID may also fail if it's removed
        # in this case, catch exception (message 404) and be quiet for a while
        # unfortunately this has changed from Py2 to Py3
        try:
            server_info = pbclient.get_server(dc_id, serverid, 1)
            server = dict(id=server_info['id'],
                          name=server_info['properties']['name'],
                          state=server_info['metadata']['state'],
                          vmstate=server_info['properties']['vmState'])
        except Exception:
            ex = sys.exc_info()[1]
            if ex.args[0] is not None and ex.args[0] == 404:
                print("Server w/ ID {} not found".format(serverid))
                server = None
            else:
                raise ex
        # end try/except
    # end if/else(serverid)
    return server
# end getServerStates()


def wait_for_server(pbclient=None, dc_id=None, serverid=None,
                    indicator='state', state='AVAILABLE', timeout=300):
    '''
    wait for a server/VM to reach a defined state for a specified time
    indicator := {state|vmstate} specifies if server or VM stat is tested
    state specifies the status the indicator should have
    '''
    if pbclient is None:
        raise ValueError("argument 'pbclient' must not be None")
    if dc_id is None:
        raise ValueError("argument 'dc_id' must not be None")
    if serverid is None:
        raise ValueError("argument 'serverid' must not be None")
    total_sleep_time = 0
    seconds = 5
    while total_sleep_time < timeout:
        time.sleep(seconds)
        total_sleep_time += seconds
        if total_sleep_time == 60:
            # Increase polling interval after one minute
            seconds = 10
        elif total_sleep_time == 600:
            # Increase polling interval after 10 minutes
            seconds = 20
        server = getServerStates(pbclient, dc_id, serverid)
        if server[indicator] == state:
            break
    # end while(total_sleep_time)
    return server
# end wait_for_server()


def main(argv=None):
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version,
                                                     program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by J. Buchhammer on %s.
  Copyright 2016 IONOS. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license,
                                formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-u', '--user', dest='user', help='the login name')
        parser.add_argument('-p', '--password', dest='password',
                            help='the login password')
        parser.add_argument('-L', '--Login', dest='loginfile', default=None,
                            help='the login file to use')
        parser.add_argument('-d', '--datacenterid', dest='dc_id',
                            required=True, default=None,
                            help='datacenter of the server')
        parser.add_argument('-s', '--serverid', dest='serverid', default=None,
                            help='ID of the server')
        parser.add_argument('-n', '--name', dest='servername', default=None,
                            help='name of the server')
        parser.add_argument('-a', '--action', dest='action', default=None,
                            required=True, help='what to do with the server')
        parser.add_argument('-C', '--command', dest='command', default=None,
                            help='remote shell command to use for shutdown')
        parser.add_argument('-v', '--verbose', dest="verbose", action="count",
                            help="set verbosity level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version',
                            version=program_version_message)

        # Process arguments
        args = parser.parse_args()
        global verbose
        verbose = args.verbose
        dc_id = args.dc_id

        if verbose > 0:
            print("Verbose mode on")

        # normalize action
        action = args.action.upper()
        actions = set(['POWERON', 'POWEROFF', 'START', 'SHUTOFF'])
        if action not in actions:
            parser.error("action must be on of {}".format(str(actions)))
        if args.serverid is None and args.servername is None:
            parser.error("one of 'serverid' or 'name' must be specified")

        (user, password) = getLogin(args.loginfile, args.user, args.password)
        if user is None or password is None:
            raise ValueError("user or password resolved to None")
        pbclient = IonosEnterpriseService(user, password)

        server = getServerStates(pbclient, dc_id,
                                 args.serverid, args.servername)
        if server is None:
            raise Exception(1, "specified server not found")
        print("using server {}(id={}) in state {}, {}"
              .format(server['name'], server['id'], server['state'],
                      server['vmstate']))
        # !!! stop/start/reboot_server() simply return 'True' !!!
        # this implies, that there's NO response nor requestId to track!
        if action == 'POWEROFF':
            if server['state'] == 'INACTIVE':
                print("server is already powered off")
            else:
                # currently use 'forced' poweroff
                if server['vmstate'] != 'SHUTOFF':
                    print("VM is in state {}, {} may lead to inconsistent state"
                          .format(server['vmstate'], action))
                    if args.command is None:
                        print("no command specified for shutdown of VM")
                    else:
                        print("executing {}".format(args.command))
                        cmdrc = call(args.command, shell=True)
                        print("executing {} returned {}"
                              .format(args.command, cmdrc))
                pbclient.stop_server(dc_id, server['id'])
                server = wait_for_server(pbclient, dc_id, server['id'],
                                         state='INACTIVE', timeout=300)
        elif action == 'POWERON':
            if server['vmstate'] == 'RUNNING':
                print("VM is already up and running")
            else:
                pbclient.start_server(dc_id, server['id'])
                server = wait_for_server(pbclient, dc_id, server['id'],
                                         indicator='vmstate', state='RUNNING',
                                         timeout=300)
        elif action == 'START':
            # this is the same as POWERON
            if server['vmstate'] == 'RUNNING':
                print("VM is already up and running")
            else:
                pbclient.start_server(dc_id, server['id'])
                server = wait_for_server(pbclient, dc_id, server['id'],
                                         indicator='vmstate', state='RUNNING',
                                         timeout=300)
        elif action == 'SHUTOFF':
            if server['vmstate'] == 'SHUTOFF':
                print("VM is already shut off")
            else:
                if args.command is None:
                    print("no command specified for shutdown of VM")
                else:
                    print("executing {}".format(args.command))
                    cmdrc = call(args.command, shell=True)
                    print("executing {} returned {}"
                          .format(args.command, cmdrc))
                    server = wait_for_server(pbclient, dc_id, server['id'],
                                             indicator='vmstate',
                                             state='SHUTOFF',
                                             timeout=300)
        # end if/else(action)
        print("server {}(id={}) now in state {}, {}"
              .format(server['name'], server['id'], server['state'],
                      server['vmstate']))
    except KeyboardInterrupt:
        # handle keyboard interrupt #
        pass
    except Exception:
        traceback.print_exc()
        sys.stderr.write("\n" + program_name + ":  for help use --help\n")
        return 2
    return 0
# end main()


if __name__ == "__main__":
    sys.exit(main())



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class snapshot:

    @IonosCoreProxy.process_response
    def get_snapshot(self, snapshot_id):
        """
        Retrieves a single snapshot by ID.

        :param      snapshot_id: The unique ID of the snapshot.
        :type       snapshot_id: ``str``

        """

        return self.get_api_instance(ionoscloud.SnapshotApi)\
            .snapshots_find_by_id_with_http_info(snapshot_id, response_type='object')

    @IonosCoreProxy.process_response
    def list_snapshots(self, depth=1):
        """
        Retrieves a list of snapshots available in the account.

        """

        return self.get_api_instance(ionoscloud.SnapshotApi)\
            .snapshots_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def delete_snapshot(self, snapshot_id):
        """
        Removes a snapshot from your account.

        :param      snapshot_id: The unique ID of the snapshot.
        :type       snapshot_id: ``str``

        """

        return self.get_api_instance(ionoscloud.SnapshotApi)\
            .snapshots_delete_with_http_info(snapshot_id)

    @IonosCoreProxy.process_response
    def update_snapshot(self, snapshot_id, **kwargs):
        """
        Removes a snapshot from your account.

        :param      snapshot_id: The unique ID of the snapshot.
        :type       snapshot_id: ``str``
        """

        return self.get_api_instance(ionoscloud.SnapshotApi)\
            .snapshots_patch_with_http_info(snapshot_id,
                                            ionoscloud.models.SnapshotProperties(**kwargs),
                                            response_type='object')

    @IonosCoreProxy.process_response
    def create_snapshot(self, datacenter_id, volume_id, name=None, description=None):
        """
        Creates a snapshot of the specified volume.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      volume_id: The unique ID of the volume.
        :type       volume_id: ``str``

        :param      name: The name given to the volume.
        :type       name: ``str``

        :param      description: The description given to the volume.
        :type       description: ``str``

        """

        return self.get_api_instance(ionoscloud.VolumeApi)\
            .datacenters_volumes_create_snapshot_post_with_http_info(
            datacenter_id, volume_id, name=name, description=description, response_type='object')

    @IonosCoreProxy.process_response
    def restore_snapshot(self, datacenter_id, volume_id, snapshot_id):
        """
        Restores a snapshot to the specified volume.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      volume_id: The unique ID of the volume.
        :type       volume_id: ``str``

        :param      snapshot_id: The unique ID of the snapshot.
        :type       snapshot_id: ``str``

        """

        return self.get_api_instance(ionoscloud.VolumeApi)\
            .datacenters_volumes_restore_snapshot_post_with_http_info(
                datacenter_id, volume_id, snapshot_id=snapshot_id)

    @IonosCoreProxy.process_response
    def remove_snapshot(self, snapshot_id):
        """
        Removes a snapshot.

        :param      snapshot_id: The ID of the snapshot
                                 you wish to remove.
        :type       snapshot_id: ``str``

        """

        return self.get_api_instance(ionoscloud.SnapshotApi)\
            .snapshots_delete_with_http_info(snapshot_id)



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

import os
"""List Datacenters
"""
from ionosenterprise.client import IonosEnterpriseService

client = IonosEnterpriseService(
    username=os.getenv('IONOS_USERNAME'), password=os.getenv('IONOS_PASSWORD'))

datacenters = client.list_datacenters()

for d in datacenters['items']:
    vdc = client.get_datacenter(d['id'])
    name = vdc['properties']['name']
    datacenter_id = vdc['id']
    break

"""Get Datacenter
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

datacenter = client.get_datacenter(
    datacenter_id=datacenter_id)

"""Create Simple Datacenter
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Datacenter, Volume, Server  # noqa

i = Datacenter(
    name='dc1',
    description='My New Datacenter',
    location='de/fkb'
)

response = client.create_datacenter(datacenter=i)

"""Create Complex Datacenter
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Datacenter, LAN, NIC, LoadBalancer, FirewallRule  # noqa

image_id = 'df8382a1-0f40-11e6-ab6b-52540005ab80'

fwrule1 = FirewallRule(
    name='Open SSH port',
    protocol='TCP',
    source_mac='01:23:45:67:89:00',
    port_range_start=22,
    port_range_end=22
)

fwrule2 = FirewallRule(
    name='Allow PING',
    protocol='ICMP',
    icmp_type=8,
    icmp_code=0
)

fw_rules = [fwrule1, fwrule2]

nic1 = NIC(
    name='nic1',
    ips=['10.2.2.3'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    firewall_rules=fw_rules
)

nic2 = NIC(
    name='nic2',
    ips=['10.2.3.4'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    firewall_rules=fw_rules
)

nics = [nic1, nic2]

volume1 = Volume(
    name='volume1',
    size=56,
    image=image_id,
    bus='VIRTIO',
    image_password="test1234"
)

volume2 = Volume(
    name='volume2',
    size=56,
    image=image_id,
    bus='VIRTIO',
    image_password="test1234"
)

volumes = [volume2]

server1 = Server(
    name='My New Server1',
    ram=4096,
    cores=4,
    nics=nics,
    create_volumes=[volume1]
)

servers = [server1]

lan1 = LAN(
    name='public Lan 4',
    public=True
)

lan2 = LAN(
    name='public Lan 5',
    public=True
)

lans = [lan1, lan2]

loadbalancer1 = LoadBalancer(
    name='LB01',
    ip='10.2.2.5',
    dhcp=False)

loadbalancers = [loadbalancer1]

d = Datacenter(
    name='My New Datacenter',
    description='Production environment',
    location='de/fkb',
    servers=servers,
    volumes=volumes,
    lans=lans,
    loadbalancers=loadbalancers
)

response = client.create_datacenter(datacenter=d)

del_response = client.delete_datacenter(response['id'])



class BackupUnit(object):
    def __init__(self, name, password=None, email=None):
        """
        BackupUnit class initializer.

        :param      name: A name of that resource (only alphanumeric characters are acceptable)"
        :type       name: ``str``

        :param      password: The password associated to that resource.
        :type       password: ``str``

        :param      email: The email associated with the backup unit.
                    Bear in mind that this email does not be the same email as of the user.
        :type       email: ``str``

        """
        self.name = name
        self.password = password
        self.email = email

    def __repr__(self):
        return ('<BackupUnit: name=%s, password=%s, email=%s>'
                % (self.name, str(self.password), self.email))



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class lan:

    @IonosCoreProxy.process_response
    def get_lan(self, datacenter_id, lan_id, depth=1):
        """
        Retrieves a single LAN by ID.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      lan_id: The unique ID of the LAN.
        :type       lan_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_find_by_id_with_http_info(
            datacenter_id, lan_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def list_lans(self, datacenter_id, depth=1):
        """
        Retrieves a list of LANs available in the account.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_get_with_http_info(
            datacenter_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def delete_lan(self, datacenter_id, lan_id):
        """
        Removes a LAN from the data center.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      lan_id: The unique ID of the LAN.
        :type       lan_id: ``str``

        """

        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_delete_with_http_info(datacenter_id, lan_id)

    @IonosCoreProxy.process_response
    def create_lan(self, datacenter_id, lan):
        """
        Creates a LAN in the data center.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      lan: The LAN object to be created.
        :type       lan: ``dict``

        """

        lan_properties = lan.__dict__
        del lan_properties['nics']
        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_post_with_http_info(datacenter_id,
                                                  ionoscloud.models.Lan(
                                                      properties=lan_properties
                                                  ),
                                                  response_type='object')

    @IonosCoreProxy.process_response
    def update_lan(self, datacenter_id, lan_id, name=None,
                   public=None, ip_failover=None, pcc=None):
        """
        Updates a LAN

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      lan_id: The unique ID of the LAN.
        :type       lan_id: ``str``

        :param      name: The new name of the LAN.
        :type       name: ``str``

        :param      public: Indicates if the LAN is public.
        :type       public: ``bool``

        :param      ip_failover: A list of IP fail-over dicts.
        :type       ip_failover: ``list``

        :param      pcc: Unique identifier of the private cross connect the given LAN is
                        connected to if any
        :type       pcc: ``str``

        """
        data = {}

        if name:
            data['name'] = name

        if public is not None:
            data['public'] = public

        if ip_failover:
            data['ipFailover'] = ip_failover

        if pcc:
            data['pcc'] = pcc

        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_patch_with_http_info(datacenter_id, lan_id,
                                                   ionoscloud.models.LanProperties(**data),
                                                   response_type='object')

    @IonosCoreProxy.process_response
    def get_lan_members(self, datacenter_id, lan_id, depth=1):
        """
        Retrieves the list of NICs that are part of the LAN.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      lan_id: The unique ID of the LAN.
        :type       lan_id: ``str``

        """

        return self.get_api_instance(ionoscloud.LanApi)\
            .datacenters_lans_nics_get_with_http_info(
            datacenter_id, lan_id, depth=depth, response_type='object')



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

"""List Snapshots
"""

from __future__ import print_function

from ionosenterprise.client import IonosEnterpriseService

client = IonosEnterpriseService(
    username='username', password='password')

snapshots = client.list_snapshots()

for s in snapshots['items']:
    print(s['properties']['name'])

"""Get Snapshot
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

snapshot_id = '7df81087-5835-41c6-a10b-3e098593bba4'

client = IonosEnterpriseService(
    username='username', password='password')

snapshot = client.get_snapshot(
    snapshot_id=snapshot_id)

"""
Update Snapshot

Valid snapshot parameters are:

* name (str)
* description (str)
* licence_type (one of 'LINUX', 'WINDOWS' or 'UNKNOWN')
* cpu_hot_plug (bool)
* ram_hot_plug (bool)
* nic_hot_plug (bool)
* nic_hot_unplug (bool)
* disc_virtio_hot_plug (bool)
* disc_virtio_hot_unplug (bool)
* disc_scsi_hot_plug (bool)
* disc_scsi_hot_unplug (bool)

"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

client = IonosEnterpriseService(
    username='username', password='password')

snapshot_id = 'd084aa0a-f9ab-41d5-9316-18163bc416ef'
image = client.update_snapshot(
    snapshot_id,
    name='New name',
    description="Backup of volume XYZ",
    licence_type='LINUX',
    cpu_hot_plug=True,
    ram_hot_plug=True,
    nic_hot_plug=True,
    nic_hot_unplug=True,
    disc_virtio_hot_plug=True,
    disc_virtio_hot_unplug=True)

"""Remove Snapshot
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa

snapshot_id = '7df81087-5835-41c6-a10b-3e098593bba4'

client = IonosEnterpriseService(
    username='username', password='password')

snapshot = client.delete_snapshot(
    snapshot_id=snapshot_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement

import os

from ionosenterprise.client import IonosEnterpriseService, Group, User

# Instantiate IonosEnterprise connection
client = IonosEnterpriseService(
    username=os.getenv('IONOS_USERNAME'),
    password=os.getenv('IONOS_PASSWORD'))

"""Create a group
"""
request = Group(
    name='demo-group',
    create_datacenter=True,
    create_snapshot=False,
    reserve_ip=True,
    access_activity_log=False)

group = client.create_group(request)

"""List groups
"""
groups = client.list_groups()

"""Create a user
"""
user_request = User(
    firstname='John',
    lastname='Doe',
    email='demo-user@example.com',
    password='SecretPassword123',
    administrator=True,
    force_sec_auth=False)

user = client.create_user(user_request)

"""List users
"""
users = client.list_users()

"""Add user to group
"""
# gu = client.add_group_user(group_id=group['id'], user_id=user['id'])
# print json.dumps(gu, indent=4)

"""List group users
"""
gus = client.list_group_users(group_id=group['id'])

"""Delete group
"""
response = client.delete_group(group['id'])

"""Delete user
"""
response = client.delete_user(user['id'])

"""List all resources
"""
# listing all resources under an admin user may take a while
resources = client.list_resources()

"""List ipblock resources
"""
ipblock_resources = client.list_resources(resource_type='ipblock')



class Group(object):
    def __init__(self, name=None, create_datacenter=None,
                 create_snapshot=None, reserve_ip=None,
                 access_activity_log=None):
        """
        Group class initializer.

        :param      name: The name of the group.
        :type       name: ``str``

        :param      create_datacenter: Indicates if the group is allowed
                                       to create virtual data centers.
        :type       create_datacenter: ``bool``

        :param      create_snapshot: Indicates if the group is allowed
                                     to create snapshots.
        :type       create_snapshot: ``bool``

        :param      reserve_ip: Indicates if the group is allowed
                                to reserve IP addresses.
        :type       reserve_ip: ``bool``

        :param      access_activity_log: Indicates if the group is allowed
                                         to access activity log.
        :type       access_activity_log: ``bool``

        """
        self.name = name
        self.create_datacenter = create_datacenter
        self.create_snapshot = create_snapshot
        self.reserve_ip = reserve_ip
        self.access_activity_log = access_activity_log

    def __repr__(self):
        return ('<Group: name=%s, create_datacenter=%s, create_snapshot=%s, '
                'reserve_ip=%s, access_activity_log=%s>'
                % (self.name, str(self.create_datacenter),
                   str(self.create_snapshot), str(self.reserve_ip),
                   str(self.access_activity_log)))



class User(object):
    def __init__(self, firstname=None, lastname=None,
                 email=None, password=None,
                 administrator=None,
                 force_sec_auth=None):
        """
        User class initializer.

        :param      firstname: The user's first name.
        :type       firstname: ``str``

        :param      lastname: The user's last name.
        :type       lastname: ``str``

        :param      email: The user's email.
        :type       email: ``str``

        :param      password: A password for the user.
        :type       password: ``str``

        :param      administrator: Indicates if the user have
                                   administrative rights.
        :type       administrator: ``bool``

        :param      force_sec_auth: Indicates if secure (two-factor)
                                    authentication should be forced
                                    for the user.
        :type       force_sec_auth: ``bool``

        """
        self.firstname = firstname
        self.lastname = lastname
        self.email = email
        self.password = password
        self.administrator = administrator
        self.force_sec_auth = force_sec_auth

    def __repr__(self):
        return ('<Group: firstname=%s, lastname=%s, email=%s, '
                'password=%s, administrator=%s, force_sec_auth=%s>'
                % (self.firstname, self.lastname,
                   self.email, self.password,
                   str(self.administrator),
                   str(self.force_sec_auth)))



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

"""List NICs
"""
from ionosenterprise.client import IonosEnterpriseService

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

nics = client.list_nics(
    datacenter_id=datacenter_id,
    server_id=server_id)

for n in nics['items']:
    print(n['properties']['name'])

"""Create NIC
"""
from ionosenterprise.client import IonosEnterpriseService, NIC  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

i = NIC(
    name='nic1',
    ips=['10.2.2.3', '10.2.3.4'],
    dhcp='true',
    lan=1,
    firewall_active=True
    )

response = client.create_nic(
    datacenter_id=datacenter_id,
    server_id=server_id,
    nic=i)

"""Create NIC with FirewallRules
"""
from ionosenterprise.client import IonosEnterpriseService, FirewallRule, NIC  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
nic_id = '<NIC-ID>'

client = IonosEnterpriseService(
    username='username', password='password')

fwrule1 = FirewallRule(
    name='Open SSH port',
    protocol='TCP',
    source_mac='01:23:45:67:89:00',
    port_range_start=22,
    port_range_end=22
    )

fwrule2 = FirewallRule(
    name='Allow PING',
    protocol='ICMP',
    icmp_type=8,
    icmp_code=0
    )

fw_rules = [fwrule1, fwrule2]

i = NIC(
    name='nic1',
    ips=['10.2.2.3', '10.2.3.4'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    firewall_rules=fw_rules
    )

response = client.create_nic(
    datacenter_id=datacenter_id,
    server_id=server_id, nic=i)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService, BackupUnit


class TestBackupunit(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        backupunitModel1 = BackupUnit("TEST 1", password="TEST PASSWORD 1",
                                      email="TEST@TEST-EMAIL1.COM")
        cls.backupunit1 = cls.client.create_backupunit(backupunitModel1)

        backupunitModel2 = BackupUnit("TEST 2", password="TEST PASSWORD 2",
                                      email="TEST@TEST-EMAIL2.COM")
        cls.backupunit2 = cls.client.create_backupunit(backupunitModel2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_backupunit(cls.backupunit1['id'])

    def test_list_backupunits(self):
        backupunits = self.client.list_backupunits()
        self.assertGreater(len(backupunits), 0)
        self.assertGreater(len(backupunits['items']), 0)

    def test_get_backupunit(self):
        backupunit = self.client.get_backupunit(self.backupunit1['id'])
        self.assertTrue('id' in backupunit)
        self.assertEqual(backupunit['type'], 'backupunit')

    def test_delete_backupunits(self):
        response = self.client.delete_backupunit(self.backupunit2['id'])
        self.assertTrue('requestId' in response)

    def test_update_backupunit(self):
        backupunit = self.client.update_backupunit(self.backupunit1['id'],
                                                   email="TEST@TEST-EMAIL-UPDATED.COM")
        self.assertEqual(backupunit['properties']['email'], "TEST@TEST-EMAIL-UPDATED.COM")

    def test_update_backupunit_put(self):
        backupunit = self.client.update_backupunit_put(self.backupunit1['id'],
                                                       email="TEST@TEST-EMAIL-UPDATED-PUT.COM")
        self.assertEqual(backupunit['properties']['email'], "TEST@TEST-EMAIL-UPDATED-PUT.COM")

    def test_get_ssourl(self):
        ssoUrl = self.client.get_ssourl(self.backupunit1['id'])
        self.assertTrue('ssoUrl' in ssoUrl)



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class resource:
    @IonosCoreProxy.process_response
    def list_resources(self, resource_type=None, depth=1):
        """
        Retrieves a list of all resources.

        :param      resource_type: The resource type: datacenter,
                                   snapshot, image, ipblock, pcc,
                                   backupunit or k8s. Default is None,
                                   i.e., all resources are listed.
        :type       resource_type: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        if resource_type is not None:
            return self.get_api_instance(ionoscloud.UserManagementApi)\
                .um_resources_find_by_type_with_http_info(resource_type, depth=depth,
                                                          response_type='object')

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_resources_get_with_http_info(depth=depth, response_type='object')

        # return response

    @IonosCoreProxy.process_response
    def get_resource(self, resource_type, resource_id, depth=1):
        """
        Retrieves a single resource of a particular type.

        :param      resource_type: The resource type: datacenter,
                                   snapshot, image, ipblock, pcc,
                                   backupunit or k8s.
        :type       resource_type: ``str``

        :param      resource_id: The unique ID of the resource.
        :type       resource_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_resources_find_by_type_and_id_with_http_info(resource_type, resource_id,
                                                             depth=depth, response_type='object')



from coreadaptor.AuthAdaptor import AuthAdaptor
from ionosenterprise import __version__
from .dicts import dicts
from .datacenter import datacenter
from .server import server
from .cdrom import cdrom
from .contract import contract
from .firewall import firewall
from .group import group
from .image import image
from .ipblock import ipblock
from .lan import lan
from .loadbalancer import loadbalancer
from .location import location
from .nic import nic
from .request import request
from .resource import resource
from .share import share
from .snapshot import snapshot
from .user import user
from .volume import volume
from .k8s import k8s
from .k8s_config import k8s_config
from .k8s_nodepools import k8s_nodepools
from .pccs import pccs
from .backupunit import backupunit
from .s3key import s3key


class IonosEnterpriseRequests(
    dicts,
    datacenter,
    server,
    cdrom,
    contract,
    firewall,
    group,
    image,
    ipblock,
    lan,
    loadbalancer,
    location,
    nic,
    request,
    resource,
    share,
    snapshot,
    user,
    volume,
    k8s,
    k8s_config,
    k8s_nodepools,
    pccs,
    backupunit,
    s3key
):
    def get_api_client(self):
        auth_adaptor = AuthAdaptor(self.username, self.password).get_api_client()
        auth_adaptor.user_agent = "ionos-cloud-sdk-python-compat/%s" % __version__
        return auth_adaptor

    def get_api_instance(self, apiClass):
        return apiClass(self.get_api_client())


__all__ = ['IonosEnterpriseRequests']



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import warnings

from random import randint
from six import assertRegex

from helpers import configuration
from helpers.resources import resource, find_image

from ionosenterprise.client import Datacenter, IPBlock, User, Group, Volume, IonosEnterpriseService
from ionosenterprise.errors import ICError, ICNotFoundError


class TestUserManagement(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create datacenter resource
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create volume resource
        volume = Volume(**cls.resource['volume'])
        cls.volume = cls.client.create_volume(
            datacenter_id=cls.datacenter['id'],
            volume=volume
        )

        cls.client.wait_for_completion(cls.volume)

        cls.image = find_image(cls.client, configuration.IMAGE_NAME)

        # Create snapshot resource
        cls.snapshot = cls.client.create_snapshot(
            datacenter_id=cls.datacenter['id'],
            volume_id=cls.volume['id'],
            name=cls.resource['snapshot']['name'])

        cls.client.wait_for_completion(cls.snapshot)

        # Reserve IP block resource
        cls.ipblock = cls.client.reserve_ipblock(IPBlock(**cls.resource['ipblock']))

        # Create User 1
        cls.user_dict1 = User(
            firstname='John',
            lastname='Doe',
            email='no-reply%s@example.com' % randint(0, 9999999999999),
            password='secretpassword123%s' % randint(0, 99999999),
            administrator=True,
            force_sec_auth=False)
        cls.user1 = cls.client.create_user(user=cls.user_dict1)

        # Create User 2
        cls.user_dict2 = User(
            firstname='John',
            lastname='Doe',
            email='no-reply%s@example.com' % randint(0, 9999999999999),
            password='secretpassword123%s' % randint(0, 99999999))
        cls.user2 = cls.client.create_user(user=cls.user_dict2)

        # Create User 3
        cls.user_dict3 = User(
            firstname='John',
            lastname='Doe',
            email='no-reply%s@example.com' % randint(0, 9999999999999),
            password='secretpassword123%s' % randint(0, 99999999))
        cls.user3 = cls.client.create_user(user=cls.user_dict3)

        # Create Group 1
        group = Group(**cls.resource['group'])
        cls.group1 = cls.client.create_group(group)

        # Create Group 2
        group.name = cls.resource['group']['name'] + ' 2'
        cls.group2 = cls.client.create_group(group)

        # Create Group 3
        group.name = cls.resource['group']['name'] + ' 3'
        cls.group3 = cls.client.create_group(group)

        # Create Share 1
        cls.share1 = cls.client.add_share(
            group_id=cls.group3['id'],
            resource_id=cls.datacenter['id'],
            edit_privilege=True,
            share_privilege=True)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_share(group_id=cls.group3['id'],
                                resource_id=cls.datacenter['id'])
        cls.client.delete_snapshot(snapshot_id=cls.snapshot['id'])
        cls.client.delete_user(user_id=cls.user1['id'])
        cls.client.delete_user(user_id=cls.user3['id'])
        cls.client.delete_group(group_id=cls.group1['id'])
        cls.client.delete_group(group_id=cls.group3['id'])
        cls.client.delete_ipblock(ipblock_id=cls.ipblock['id'])
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_create_user(self):
        self.assertEqual(self.user1['type'], 'user')
        self.assertEqual(self.user1['properties']['firstname'], self.user_dict1.firstname)
        self.assertEqual(self.user1['properties']['lastname'], self.user_dict1.lastname)
        self.assertEqual(self.user1['properties']['email'], self.user_dict1.email)
        self.assertEqual(self.user1['properties']['administrator'], self.user_dict1.administrator)
        self.assertEqual(self.user1['properties']['forceSecAuth'], self.user_dict1.force_sec_auth)

    def test_list_users(self):
        users = self.client.list_users()

        self.assertGreater(len(users['items']), 0)
        self.assertEqual(users['items'][0]['type'], 'user')

    def test_get_user(self):
        user = self.client.get_user(user_id=self.user1['id'])

        self.assertEqual(user['type'], 'user')
        self.assertEqual(user['id'], self.user1['id'])
        self.assertEqual(user['properties']['firstname'], self.user1['properties']['firstname'])
        self.assertEqual(user['properties']['lastname'], self.user1['properties']['lastname'])
        self.assertEqual(user['properties']['email'], self.user1['properties']['email'])
        self.assertEqual(user['properties']['administrator'],
                         self.user1['properties']['administrator'])
        self.assertEqual(user['properties']['forceSecAuth'],
                         self.user1['properties']['forceSecAuth'])
        self.assertFalse(user['properties']['secAuthActive'])

    def test_delete_user(self):
        user = self.client.delete_user(user_id=self.user2['id'])

        self.assertTrue(user)
        assertRegex(self, user['requestId'], self.resource['uuid_match'])

    def test_update_user(self):
        user = self.client.update_user(
            user_id=self.user1['id'],
            firstname=self.user1['properties']['firstname'],
            lastname=self.user1['properties']['lastname'],
            email=self.user1['properties']['email'],
            administrator=False,
            force_sec_auth=self.user1['properties']['forceSecAuth']
        )

        self.assertEqual(user['type'], 'user')
        self.assertEqual(user['id'], self.user1['id'])
        self.assertEqual(user['properties']['firstname'], self.user1['properties']['firstname'])
        self.assertEqual(user['properties']['lastname'], self.user1['properties']['lastname'])
        self.assertEqual(user['properties']['email'], self.user1['properties']['email'])
        self.assertFalse(user['properties']['administrator'])
        self.assertEqual(user['properties']['forceSecAuth'],
                         self.user1['properties']['forceSecAuth'])

    def test_create_user_failure(self):
        try:
            user = User(
                firstname='John',
                lastname='Doe',
                password='secretpassword123')
            self.client.create_user(user)
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'email',
                          e.content[0]['message'])

    def test_get_user_failure(self):
        try:
            self.client.get_user('00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_group(self):
        self.assertEqual(self.group1['type'], 'group')
        self.assertEqual(self.group1['properties']['name'], self.resource['group']['name'])
        self.assertEqual(self.group1['properties']['createDataCenter'],
                         self.resource['group']['create_datacenter'])
        self.assertEqual(self.group1['properties']['createSnapshot'],
                         self.resource['group']['create_snapshot'])
        self.assertEqual(self.group1['properties']['reserveIp'],
                         self.resource['group']['reserve_ip'])
        self.assertEqual(self.group1['properties']['accessActivityLog'],
                         self.resource['group']['access_activity_log'])

    def test_list_groups(self):
        groups = self.client.list_groups()

        self.assertGreater(len(groups['items']), 0)
        self.assertEqual(groups['items'][0]['type'], 'group')

    def test_get_group(self):
        group = self.client.get_group(group_id=self.group1['id'])

        self.assertEqual(group['type'], 'group')
        self.assertEqual(group['id'], self.group1['id'])
        self.assertEqual(group['properties']['name'], self.group1['properties']['name'])
        self.assertEqual(group['properties']['createDataCenter'],
                         self.group1['properties']['createDataCenter'])
        self.assertEqual(group['properties']['createSnapshot'],
                         self.group1['properties']['createSnapshot'])
        self.assertEqual(group['properties']['reserveIp'],
                         self.group1['properties']['reserveIp'])
        self.assertEqual(group['properties']['accessActivityLog'],
                         self.group1['properties']['accessActivityLog'])

    def test_update_group(self):
        group = self.client.update_group(
            group_id=self.group1['id'],
            name=self.resource['group']['name'] + ' - RENAME',
            create_datacenter=False
        )

        self.assertEqual(group['type'], 'group')
        self.assertEqual(group['id'], self.group1['id'])
        self.assertEqual(group['properties']['name'],
                         self.resource['group']['name'] + ' - RENAME')
        self.assertFalse(group['properties']['createDataCenter'])

    def test_delete_group(self):
        group = self.client.delete_group(group_id=self.group2['id'])

        self.assertTrue(group)
        assertRegex(self, group['requestId'], self.resource['uuid_match'])

    def test_create_group_failure(self):
        try:
            self.client.create_group(Group(create_datacenter=True))
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'name',
                          e.content[0]['message'])

    def test_get_group_failure(self):
        try:
            self.client.get_group('00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_add_share(self):
        self.assertEqual(self.share1['type'], 'resource')
        self.assertTrue(self.share1['properties']['editPrivilege'])
        self.assertTrue(self.share1['properties']['sharePrivilege'])

    def test_list_shares(self):
        shares = self.client.list_shares(group_id=self.group3['id'])

        self.assertGreater(len(shares['items']), 0)
        self.assertEqual(shares['items'][0]['type'], 'resource')

    def test_get_share(self):
        share = self.client.get_share(group_id=self.group3['id'],
                                      resource_id=self.datacenter['id'])

        self.assertEqual(share['id'], self.datacenter['id'])
        self.assertEqual(share['type'], 'resource')
        self.assertTrue(share['properties']['editPrivilege'])
        self.assertTrue(share['properties']['sharePrivilege'])

    def test_update_share(self):
        share = self.client.update_share(group_id=self.group3['id'],
                                         resource_id=self.datacenter['id'],
                                         share_privilege=False)

        self.assertEqual(share['id'], self.datacenter['id'])
        self.assertEqual(share['type'], 'resource')
        self.assertFalse(share['properties']['sharePrivilege'])

    def test_get_share_failure(self):
        try:
            self.client.get_share(group_id=self.group3['id'],
                                  resource_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_share_failure(self):
        try:
            self.client.add_share(group_id=self.group3['id'],
                                  resource_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_list_group_users(self):
        users = self.client.list_group_users(group_id=self.group3['id'])

        self.assertGreater(len(users['items']), 0)
        self.assertEqual(users['items'][0]['type'], 'user')

    def test_add_group_user(self):
        user = self.client.add_group_user(group_id=self.group3['id'],
                                          user_id=self.user3['id'])

        self.assertEqual(user['id'], self.user3['id'])
        self.assertEqual(user['type'], 'user')

    def test_remove_group_user(self):
        user = self.client.remove_group_user(group_id=self.group3['id'],
                                             user_id=self.user3['id'])

        self.assertTrue(user)

    def test_list_resources(self):
        resources = self.client.list_resources()

        self.assertGreater(len(resources['items']), 0)
        self.assertEqual(resources['id'], 'resources')
        self.assertEqual(resources['type'], 'collection')

    def test_list_datacenter_resources(self):
        resources = self.client.list_resources(resource_type='datacenter')

        self.assertGreater(len(resources['items']), 0)
        self.assertEqual(resources['id'], 'resources')
        self.assertEqual(resources['items'][0]['type'], 'datacenter')

    def test_list_image_resources(self):
        resources = self.client.list_resources(resource_type='image')

        self.assertEqual(resources['id'], 'resources')

    def test_list_snapshot_resources(self):
        resources = self.client.list_resources(resource_type='snapshot')

        self.assertGreater(len(resources['items']), 0)
        self.assertEqual(resources['id'], 'resources')
        self.assertEqual(resources['items'][0]['type'], 'snapshot')

    def test_list_ipblock_resources(self):
        resources = self.client.list_resources(resource_type='ipblock')

        self.assertGreater(len(resources['items']), 0)
        self.assertEqual(resources['id'], 'resources')
        self.assertEqual(resources['items'][0]['type'], 'ipblock')

    def test_list_resources_failure(self):
        try:
            self.client.list_resources(resource_type='unknown')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_get_datacenter_resource(self):
        data_center = self.client.get_resource(resource_type='datacenter',
                                               resource_id=self.datacenter['id'])

        self.assertEqual(data_center['id'], self.datacenter['id'])
        self.assertEqual(data_center['type'], 'datacenter')

    def test_get_image_resource(self):
        image = self.client.get_resource(resource_type='image',
                                         resource_id=self.image['id'])

        self.assertEqual(image['id'], self.image['id'])
        self.assertEqual(image['type'], 'image')

    def test_get_snapshot_resource(self):
        snapshot = self.client.get_resource(resource_type='snapshot',
                                            resource_id=self.snapshot['id'])

        self.assertEqual(snapshot['id'], self.snapshot['id'])
        self.assertEqual(snapshot['type'], 'snapshot')

    def test_list_ipblock_resources2(self):
        ipblock = self.client.get_resource(resource_type='ipblock',
                                           resource_id=self.ipblock['id'])

        self.assertEqual(ipblock['id'], self.ipblock['id'])
        self.assertEqual(ipblock['type'], 'ipblock')

    def test_get_resource_failure(self):
        try:
            self.client.get_resource(resource_type='datacenter',
                                     resource_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=reimported,wrong-import-position

"""Create Simple Server
"""

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.client import Server

server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

i = Server(
    name='server',
    ram=4096,
    cores=4
    )

response = client.create_server(
    datacenter_id=datacenter_id,
    server=i)

"""Create Complex Server
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server, NIC, Volume  # noqa

server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
image_id = '226ed8c0-a2fe-11e4-b187-5f1f641608c8'

client = IonosEnterpriseService(
    username='username', password='password')

nic1 = NIC(
    name='nic1',
    ips=['10.2.2.5'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

nic2 = NIC(
    name='nic2',
    ips=['10.2.3.6'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

volume1 = Volume(
    name='volume6',
    size=56,
    image=image_id,
    bus='VIRTIO'
    )

volume2 = Volume(
    name='volume7',
    size=56,
    image=image_id,
    bus='VIRTIO'
    )

nics = [nic1, nic2]
create_volumes = [volume1, volume2]

i = Server(
    name='server11',
    ram=4096,
    cores=4,
    nics=nics,
    create_volumes=create_volumes
    )

response = client.create_server(
    datacenter_id=datacenter_id, server=i)

"""Create Server with Existing Volume
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server  # noqa

server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id = '700e1cab-99b2-4c30-ba8c-1d273ddba044'

client = IonosEnterpriseService(
    username='username', password='password')

attach_volumes = [volume_id]

i = Server(
    name='server1',
    ram=4096,
    cores=4,
    attach_volumes=attach_volumes
    )

response = client.create_server(
    datacenter_id=datacenter_id, server=i)

"""Create Server with New Volumes
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server, Volume  # noqa

server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
volume_id = '700e1cab-99b2-4c30-ba8c-1d273ddba044'

datacenter_id = 'c4fd1f8a-65e0-42cb-b8fa-ff7e87c3071b'
image_id = '27500669-d81b-11e4-aea4-52540066fee9'

client = IonosEnterpriseService(
    username='username', password='password')

volume1 = Volume(
    name='volume11',
    size=56,
    image=image_id,
    bus='VIRTIO'
    )

volume2 = Volume(
    name='volume21',
    size=56,
    image=image_id,
    bus='VIRTIO'
    )

create_volumes = [volume1, volume2]

i = Server(
    name='server12',
    ram=4096,
    cores=4,
    create_volumes=create_volumes
    )

response = client.create_server(
    datacenter_id=datacenter_id, server=i)

"""Create Server with NICs Only
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server, NIC  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

nic1 = NIC(
    name='nic1',
    ips=['10.2.2.3'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

nic2 = NIC(
    name='nic2',
    ips=['10.2.3.4'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

nics = [nic1, nic2]

i = Server(
    name='server87',
    ram=4096,
    cores=4,
    nics=nics
    )

response = client.create_server(
    datacenter_id=datacenter_id, server=i)

"""Create Server with Two Existing Volumes
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id1 = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
volume_id2 = '800e1cab-99b2-4c30-ba8c-1d273ddba024'

client = IonosEnterpriseService(
    username='username', password='password')

attach_volumes = [volume_id1, volume_id2]

i = Server(
    name='server1',
    ram=4096,
    cores=4,
    attach_volumes=attach_volumes
    )

response = client.create_server(
    datacenter_id=datacenter_id,
    server=i)

"""Create Server with Boot Volume
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id1 = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
volume_id2 = '800e1cab-99b2-4c30-ba8c-1d273ddba024'
boot_volume_id = '800e1cab-99b2-4c30-ba8c-1d273ddba024'

client = IonosEnterpriseService(
    username='username', password='password')

i = Server(
    name='server14',
    ram=4096,
    cores=4,
    boot_volume_id=boot_volume_id
    )

response = client.create_server(
    datacenter_id=datacenter_id,
    server=i)

"""Create Server with Existing Volumes and NICs
"""

from ionosenterprise.client import IonosEnterpriseService  # noqa
from ionosenterprise.client import Server, NIC  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
volume_id1 = '700e1cab-99b2-4c30-ba8c-1d273ddba023'
volume_id2 = '800e1cab-99b2-4c30-ba8c-1d273ddba024'
boot_volume_id = '800e1cab-99b2-4c30-ba8c-1d273ddba024'

client = IonosEnterpriseService(
    username='username', password='password')

attach_volumes = [volume_id1, volume_id2]

nic1 = NIC(
    name='nic1',
    ips=['10.2.2.3'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

nic2 = NIC(
    name='nic2',
    ips=['10.2.3.4'],
    dhcp='true',
    lan=1,
    firewall_active=True,
    )

nics = [nic1, nic2]

i = Server(
    name='server1',
    ram=4096,
    cores=4,
    boot_volume_id=boot_volume_id,
    attach_volumes=attach_volumes,
    nics=nics
    )

response = client.create_server(
    datacenter_id=datacenter_id,
    server=i)

"""Start Server
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

server = client.start_server(
    datacenter_id=datacenter_id,
    server_id=server_id)

"""Stop Server
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

server = client.stop_server(
    datacenter_id=datacenter_id,
    server_id=server_id)

"""Reboot Server
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

server = client.reboot_server(
    datacenter_id=datacenter_id,
    server_id=server_id)

"""List Servers
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'

client = IonosEnterpriseService(
    username='username', password='password')

servers = client.list_servers(datacenter_id=datacenter_id)

"""Delete Server
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

server = client.delete_server(
    datacenter_id=datacenter_id,
    server_id=server_id)

"""Update Server
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

datacenter_id = '700e1cab-99b2-4c30-ba8c-1d273ddba022'
server_id = '700e1cab-99b2-4c30-ba8c-1d273ddba023'

client = IonosEnterpriseService(
    username='username', password='password')

server = client.update_server(
    datacenter_id=datacenter_id,
    server_id=server_id,
    cores=35,
    ram=2048)



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

import os
"""List IPBlocks
"""
from ionosenterprise.client import IonosEnterpriseService

client = IonosEnterpriseService(
    username=os.getenv('IONOS_USERNAME'), password=os.getenv('IONOS_PASSWORD'))

ipblocks = client.list_ipblocks()

print(ipblocks)

"""Reserve IPBlock
"""
from ionosenterprise.client import IonosEnterpriseService, IPBlock  # noqa

i = IPBlock(name='py-test', location='de/fra', size=1)

ipblock = client.reserve_ipblock(i)

"""Release IPBlock
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

ipblock_id = ipblock['id']


ipblock = client.delete_ipblock(ipblock_id)



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class k8s_nodepools:

    @IonosCoreProxy.process_response
    def get_k8s_cluster_nodepool(self, k8s_cluster_id, nodepool_id):
        """
        Retrieve Kubernetes Node Pool

        :param      k8s_cluster_id: The unique ID of the Kubernetes Cluster
        :type       k8s_cluster_id: ``str``

        :param      nodepool_id: The unique ID of the Kubernetes Node Pool
        :type       nodepool_id: ``str``

        """
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_nodepools_find_by_id_with_http_info(
            k8s_cluster_id, nodepool_id, response_type='object')

    @IonosCoreProxy.process_response
    def delete_k8s_cluster_nodepool(self, k8s_cluster_id, nodepool_id):
        """
        Delete Kubernetes Node Pool

        :param      k8s_cluster_id: The unique ID of the Kubernetes Cluster
        :type       k8s_cluster_id: ``str``

        :param      nodepool_id: The unique ID of the Kubernetes Node Pool
        :type       nodepool_id: ``str``

        """

        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_nodepools_delete_with_http_info(k8s_cluster_id, nodepool_id)

    @IonosCoreProxy.process_response
    def update_k8s_cluster_nodepool(self,
                                    k8s_cluster_id, nodepool_id, node_count,
                                    maintenance_window=None, auto_scaling=None,
                                    lan_ids=None, public_ips=None):
        """
        This will modify the Kubernetes Node Pool.
        :param      k8s_cluster_id: The unique ID of the Kubernetes Cluster
        :type       k8s_cluster_id: ``str``
        :param      nodepool_id: The unique ID of the Kubernetes Node Pool
        :type       nodepool_id: ``str``
        :param      node_count: Number of nodes part of the Node Pool
        :type       node_count: ``int``
        :param      maintenance_window: MaintenanceWindow
        :type       maintenance_window: ``dict, e.g. {'dayOfTheWeek':'', 'time':''}``
                        dayOfTheWeek: The day of the week for a maintenance window.
                        dayOfTheWeek: ``str`` ["Monday",
                                            "Tuesday",
                                            "Wednesday",
                                            "Thursday",
                                            "Friday",
                                            "Saturday",
                                            "Sunday"]
                        time: The time to use for a maintenance window. Accepted formats are:
                        HH:mm:ss; HH:mm:ss"Z";
                            HH:mm:ssZ. This time may varies by 15 minutes.
                        time: ``string``
        :param      auto_scaling: AutoScaling object
        :type       auto_scaling: ``dict`` {'minNodeCount': 2, 'maxNodeCount': 3}
                        minNodeCount": The minimum number of worker nodes that the managed node
                         group can scale in.
                            Should be set together with 'maxNodeCount'.
                            Value for this attribute must be greater than equal to 1 and less
                            than equal to maxNodeCount.
                        minNodeCount" : ``integer``
                        maxNodeCount: The maximum number of worker nodes that the managed node
                        pool can scale-out.
                                Should be set together with 'minNodeCount'.
                                Value for this attribute must be greater than equal to 1 and
                                minNodeCount.
                        maxNodeCount: ``integer``

        :param      lan_ids: array of additional LANs attached to worker nodes
        :type       lan_ids: ``list of ints``

        :param      public_ips: Optional array of reserved public IP addresses to be used by
                        the nodes.
                        IPs must be from same location as the data center used for the node pool.
                        The array must contain one extra IP than maximum number of nodes could be.
                        (nodeCount+1 if fixed node amount or maxNodeCount+1 if auto scaling is
                        used).
                        The extra provided IP Will be used during rebuilding of nodes.
        :type       public_ips: ``list``
        """

        properties = {node_count: node_count}

        if maintenance_window is not None:
            properties['maintenance_window'] = maintenance_window
        if auto_scaling is not None:
            properties['auto_scaling'] = auto_scaling
        if lan_ids is not None:
            properties['lans'] = [{'id': lan_id} for lan_id in lan_ids]
        if public_ips is not None:
            properties['public_ips'] = public_ips

        kubernetesNodePool = ionoscloud.models.KubernetesNodePool(
            properties=ionoscloud.models.KubernetesNodePoolProperties(
                **properties
            )
        )
        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_nodepools_put_with_http_info(k8s_cluster_id,
                                              nodepool_id,
                                              kubernetesNodePool,
                                              response_type='object')

    @IonosCoreProxy.process_response
    def list_k8s_cluster_nodepools(self, k8s_cluster_id, depth=1):
        """
        List Kubernetes Node Pools

        :param      k8s_cluster_id: The unique ID of the Kubernetes Cluster
        :type       k8s_cluster_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_nodepools_get_with_http_info(k8s_cluster_id, depth=depth,
                                              response_type='object')

    @IonosCoreProxy.process_response
    def create_k8s_cluster_nodepool(self,
                                    k8s_cluster_id,
                                    name, datacenter_id,
                                    node_count, cpu_family,
                                    cores_count, ram_size,
                                    availability_zone,
                                    storage_type, storage_size,
                                    k8s_version=None, maintenance_window=None, auto_scaling=None,
                                    lan_ids=None, labels=None, annotations=None, public_ips=None):
        """
        This will create a new Kubernetes Node Pool inside a Kubernetes Cluster.
        :param      k8s_cluster_id: The unique ID of the Kubernetes Cluster
        :type       k8s_cluster_id: ``str``
        :param      nodepool_id: The unique ID of the Kubernetes Node Pool
        :type       nodepool_id: ``str``
        :param      name: A Kubernetes Node Pool Name. Valid Kubernetes
                          Node Pool name must be 63 characters or less
                          and must be empty or begin and end with an
                          alphanumeric character ([a-z0-9A-Z]) with
                          dashes (-), underscores (_), dots (.), and
                          alphanumerics between.
        :type       name: ``str``
        :param      datacenter_id: A valid uuid of the datacenter on which user has access
        :type       datacenter_id: ``str``
        :param      node_count: Number of nodes part of the Node Pool
        :type       node_count: ``int``
        :param      cpu_family: A valid cpu family name
        :type       cpu_family: ``str``
        :param      cores_count: Number of cores for node
        :type       cores_count: ``int``
        :param      ram_size: RAM size for node, minimum size 2048MB is recommended
        :type       ram_size: ``int``
        :param      availability_zone: The availability zone in which the server should exist
        :type       availability_zone: ``str``
        :param      storage_type: Hardware type of the volume
        :type       storage_type: ``str``
        :param      storage_size: The size of the volume in GB. The size
                                  should be greater than 10GB
        :type       storage_size: ``int``
        :param      k8s_version: Kubernetes version
        :type       k8s_version: ``str``
        :param      maintenance_window: MaintenanceWindow
        :type       maintenance_window: ``dict, e.g. {'dayOfTheWeek':'', 'time':''}``
                        dayOfTheWeek: The day of the week for a maintenance window.
                        dayOfTheWeek: ``str`` ["Monday",
                                            "Tuesday",
                                            "Wednesday",
                                            "Thursday",
                                            "Friday",
                                            "Saturday",
                                            "Sunday"]
                        time: The time to use for a maintenance window. Accepted formats
                            are: HH:mm:ss; HH:mm:ss"Z";
                            HH:mm:ssZ. This time may varies by 15 minutes.
                        time: ``string``
        :param      auto_scaling: AutoScaling object
        :type       auto_scaling: ``dict`` {'minNodeCount': 2, 'maxNodeCount': 3}
                        minNodeCount": The minimum number of worker nodes that the managed node
                         group can scale in.
                            Should be set together with 'maxNodeCount'.
                            Value for this attribute must be greater than equal to 1 and less than
                             equal to
                            maxNodeCount.
                        minNodeCount" : ``integer``
                        maxNodeCount: The maximum number of worker nodes that the managed node pool
                        can scale-out.
                                Should be set together with 'minNodeCount'.
                                Value for this attribute must be greater than equal to 1 and
                                minNodeCount.
                        maxNodeCount: ``integer``
        :param      lan_ids: array of additional LANs attached to worker nodes
        :type       lan_ids: ``list of ints``
        :param      labels: map of labels attached to node pool
        :type       labels: ``dict``
        :param      annotations: map of annotations attached to node pool
        :type       annotations: ``dict``
        :param      public_ips: Optional array of reserved public IP addresses to be used by the
                        nodes.
                        IPs must be from same location as the data center used for the node pool.
                        The array must contain one extra IP than maximum number of nodes could be.
                        (nodeCount+1 if fixed node amount or maxNodeCount+1 if auto scaling is
                        used).
                        The extra provided IP Will be used during rebuilding of nodes.
        :type       public_ips: ``list``
        """

        # mandatory fields
        properties = {
            'name': name,
            'datacenter_id': datacenter_id,
            'node_count': node_count,
            'cpu_family': cpu_family,
            'cores_count': cores_count,
            'ram_size': ram_size,
            'availability_zone': availability_zone,
            'storage_type': storage_type,
            'storage_size': storage_size
        }

        # optional fields
        if k8s_version is not None:
            properties['k8s_version'] = k8s_version
        if maintenance_window is not None:
            properties['maintenance_window'] = maintenance_window
        if auto_scaling is not None:
            properties['auto_scaling'] = auto_scaling
        if lan_ids is not None:
            properties['lans'] = [{'id': int(lan_id)} for lan_id in lan_ids]
        if labels is not None:
            properties['labels'] = labels
        if annotations is not None:
            properties['annotations'] = annotations
        if public_ips is not None:
            properties['public_ips'] = public_ips

        kubernetes_node_pool = ionoscloud.models.KubernetesNodePool(
            properties=ionoscloud.models.KubernetesNodePoolProperties(
                **properties
            )
        )

        return self.get_api_instance(ionoscloud.KubernetesApi)\
            .k8s_nodepools_post_with_http_info(k8s_cluster_id, kubernetes_node_pool,
                                               response_type='object')



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class user:

    @IonosCoreProxy.process_response
    def list_users(self, depth=1):
        """
        Retrieves a list of all users.

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_users_get_with_http_info(depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def get_user(self, user_id, depth=1):
        """
        Retrieves a single user by ID.

        :param      user_id: The unique ID of the user.
        :type       user_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_users_find_by_id_with_http_info(user_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def create_user(self, user):
        """
        Creates a new user.

        :param      user: The user object to be created.
        :type       user: ``dict``

        """
        data = self._create_user_dict(user=user)

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_users_post_with_http_info(ionoscloud.models.User(**data), response_type='object')

    @IonosCoreProxy.process_response
    def update_user(self, user_id, **kwargs):
        """
        Updates a user.

        :param      user_id: The unique ID of the user.
        :type       user_id: ``str``

        """
        properties = {}

        for attr, value in kwargs.items():
            properties[self._underscore_to_camelcase(attr)] = value

        user = ionoscloud.models.User(properties=properties)
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_users_put_with_http_info(user_id, user, response_type='object')

    @IonosCoreProxy.process_response
    def delete_user(self, user_id):
        """
        Removes a user.

        :param      user_id: The unique ID of the user.
        :type       user_id: ``str``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_users_delete_with_http_info(user_id)

    @IonosCoreProxy.process_response
    def list_group_users(self, group_id, depth=1):
        """
        Retrieves a list of all users that are members of a particular group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_users_get_with_http_info(group_id, depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def add_group_user(self, group_id, user_id):
        """
        Adds an existing user to a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      user_id: The unique ID of the user.
        :type       user_id: ``str``

        """

        user = ionoscloud.models.User(id=user_id)
        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_users_post_with_http_info(group_id, user, response_type='object')

    @IonosCoreProxy.process_response
    def remove_group_user(self, group_id, user_id):
        """
        Removes a user from a group.

        :param      group_id: The unique ID of the group.
        :type       group_id: ``str``

        :param      user_id: The unique ID of the user.
        :type       user_id: ``str``

        """

        return self.get_api_instance(ionoscloud.UserManagementApi)\
            .um_groups_users_delete_with_http_info(group_id, user_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import Datacenter, Server, LAN, NIC, IonosEnterpriseService
from ionosenterprise.errors import ICError, ICNotFoundError


class TestNic(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.client.wait_for_completion(cls.datacenter)

        # Create test LAN.
        cls.lan = cls.client.create_lan(
            datacenter_id=cls.datacenter['id'],
            lan=LAN(name=cls.resource['lan']['name'], public=False))
        cls.client.wait_for_completion(cls.lan)

        # Create test server.
        cls.server = cls.client.create_server(
            datacenter_id=cls.datacenter['id'],
            server=Server(**cls.resource['server']))
        cls.client.wait_for_completion(cls.server)

        # Create test NIC1.
        nic1 = NIC(**cls.resource['nic'])
        nic1.lan = cls.lan['id']
        cls.ips = ['10.0.0.1']
        nic1.ips = cls.ips
        cls.nic1 = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=nic1)
        cls.client.wait_for_completion(cls.nic1)

        # Create test NIC2.
        nic2 = NIC(**cls.resource['nic'])
        nic2.lan = cls.lan['id']
        cls.nic2 = cls.client.create_nic(
            datacenter_id=cls.datacenter['id'],
            server_id=cls.server['id'],
            nic=nic2)
        cls.client.wait_for_completion(cls.nic2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_nics(self):
        nics = self.client.list_nics(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'])

        self.assertGreater(len(nics), 0)
        self.assertIn(nics['items'][0]['id'], (self.nic1['id'], self.nic2['id']))
        self.assertEqual(nics['items'][0]['type'], 'nic')

    def test_get_nic(self):
        nic = self.client.get_nic(datacenter_id=self.datacenter['id'],
                                  server_id=self.server['id'],
                                  nic_id=self.nic1['id'])

        self.assertEqual(nic['type'], 'nic')
        self.assertEqual(nic['id'], self.nic1['id'])
        assertRegex(self, nic['id'], self.resource['uuid_match'])
        self.assertEqual(nic['properties']['name'], self.resource['nic']['name'])
        self.assertEqual(nic['properties']['firewallActive'],
                         self.resource['nic']['firewall_active'])
        self.assertIsInstance(nic['properties']['ips'], list)
        self.assertEqual(nic['properties']['dhcp'], self.resource['nic']['dhcp'])
        self.assertEqual(nic['properties']['nat'], self.resource['nic']['nat'])
        self.assertEqual(nic['properties']['lan'], self.resource['nic']['lan'])

    def test_delete_nic(self):
        nic2 = self.client.delete_nic(datacenter_id=self.datacenter['id'],
                                      server_id=self.server['id'],
                                      nic_id=self.nic2['id'])

        self.assertTrue(nic2)
        assertRegex(self, nic2['requestId'], self.resource['uuid_match'])

    def test_update_nic(self):
        nic = self.client.update_nic(
            datacenter_id=self.datacenter['id'],
            server_id=self.server['id'],
            nic_id=self.nic1['id'],
            name=self.resource['nic']['name'] + ' - RENAME')

        self.assertEqual(nic['id'], self.nic1['id'])
        self.assertEqual(nic['type'], 'nic')
        self.assertEqual(nic['properties']['name'], self.resource['nic']['name'] + ' - RENAME')

    def test_create_nic(self):
        self.assertEqual(self.nic1['type'], 'nic')
        self.assertEqual(self.nic1['properties']['name'], self.resource['nic']['name'])
        self.assertEqual(self.nic1['properties']['firewallActive'],
                         self.resource['nic']['firewall_active'])
        self.assertIsInstance(self.nic1['properties']['ips'], list)
        self.assertEqual(self.nic1['properties']['dhcp'], self.resource['nic']['dhcp'])
        self.assertIsNone(self.nic1['properties']['nat'])
        self.assertEqual(str(self.nic1['properties']['lan']), self.lan['id'])

    def test_get_failure(self):
        try:
            self.client.get_nic(
                datacenter_id=self.datacenter['id'],
                server_id=self.server['id'],
                nic_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_failure(self):
        try:
            nic = NIC(name=self.resource['nic']['name'])
            self.client.create_nic(
                datacenter_id=self.datacenter['id'],
                server_id=self.server['id'],
                nic=nic)
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'lan',
                          e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()



import ionoscloud
from coreadaptor.IonosCoreProxy import IonosCoreProxy


class nic:

    @IonosCoreProxy.process_response
    def get_nic(self, datacenter_id, server_id, nic_id, depth=1):
        """
        Retrieves a NIC by its ID.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.NicApi)\
            .datacenters_servers_nics_find_by_id_with_http_info(
                datacenter_id,
                server_id,
                nic_id,
                depth=depth,
                response_type='object'
            )

    @IonosCoreProxy.process_response
    def list_nics(self, datacenter_id, server_id, depth=1):
        """
        Retrieves a list of all NICs bound to the specified server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      depth: The depth of the response data.
        :type       depth: ``int``

        """
        return self.get_api_instance(ionoscloud.NicApi)\
            .datacenters_servers_nics_get_with_http_info(datacenter_id, server_id,
                                                         depth=depth, response_type='object')

    @IonosCoreProxy.process_response
    def delete_nic(self, datacenter_id, server_id, nic_id):
        """
        Removes a NIC from the server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        """

        return self.get_api_instance(ionoscloud.NicApi)\
            .datacenters_servers_nics_delete_with_http_info(datacenter_id, server_id, nic_id)

    @IonosCoreProxy.process_response
    def create_nic(self, datacenter_id, server_id, nic):
        """
        Creates a NIC on the specified server.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic: A NIC dict.
        :type       nic: ``dict``

        """

        return self.get_api_instance(ionoscloud.NicApi)\
            .datacenters_servers_nics_post_with_http_info(datacenter_id,
                                                          server_id,
                                                          ionoscloud.models.Nic(
                                                              **self._create_nic_dict(nic)
                                                          ),
                                                          response_type='object')

    @IonosCoreProxy.process_response
    def update_nic(self, datacenter_id, server_id, nic_id, **kwargs):
        """
        Updates a NIC with the parameters provided.

        :param      datacenter_id: The unique ID of the data center.
        :type       datacenter_id: ``str``

        :param      server_id: The unique ID of the server.
        :type       server_id: ``str``

        :param      nic_id: The unique ID of the NIC.
        :type       nic_id: ``str``

        """
        data = {}

        for attr, value in kwargs.items():
            data[self._underscore_to_camelcase(attr)] = value

        return self.get_api_instance(ionoscloud.NicApi)\
            .datacenters_servers_nics_patch_with_http_info(datacenter_id, server_id, nic_id,
                                                           data, response_type='object')



class NIC(object):
    def __init__(self, name=None, ips=None,  # pylint: disable=unused-argument
                 dhcp=None, lan=None, firewall_active=None,
                 firewall_rules=None, nat=None):
        """
        NIC class initializer.

        :param      name: The name of the NIC.
        :type       name: ``str``

        :param      ips: A list of IPs.
        :type       ips: ``list``

        :param      dhcp: Enable or disable DHCP. Default is enabled.
        :type       dhcp: ``bool``

        :param      lan: ID of the LAN in which the NIC should reside.
        :type       lan: ``str``

        :param      nat: Enable or disable NAT. Default is disabled.
        :type       nat: ``bool``

        :param      firewall_active: Turns the firewall on or off;
                                     default is disabled.
        :type       firewall_active: ``bool``

        :param      firewall_rules: List of firewall rule dicts.
        :type       firewall_rules: ``list``

        """
        if firewall_rules is None:
            firewall_rules = []
        self.name = name
        self.nat = nat
        self.ips = ips
        self.dhcp = dhcp
        self.lan = lan
        self.firewall_active = firewall_active
        self.firewall_rules = firewall_rules

    def __repr__(self):
        return (('<NIC: name=%s, ips=%s, dhcp=%s,lan=%s, '
                 'firewall_active=%s> ...>')
                % (self.name, self.ips, str(self.dhcp),
                   self.lan, str(self.firewall_active)))



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest
import time

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import Datacenter, IonosEnterpriseService, Server, Volume
from ionosenterprise.errors import ICError, ICNotFoundError


class TestDatacenter(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test datacenter.
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_datacenter(datacenter_id=cls.datacenter['id'])

    def test_list_datacenters(self):
        datacenters = self.client.list_datacenters()

        self.assertGreater(len(datacenters), 0)
        self.assertEqual(datacenters['items'][0]['type'], 'datacenter')

    def test_get_datacenter(self):
        datacenter = self.client.get_datacenter(
            datacenter_id=self.datacenter['id'])

        assertRegex(self, datacenter['id'], self.resource['uuid_match'])
        self.assertEqual(datacenter['type'], 'datacenter')
        self.assertEqual(datacenter['id'], self.datacenter['id'])
        self.assertEqual(datacenter['properties']['name'], self.resource['datacenter']['name'])
        self.assertEqual(datacenter['properties']['description'],
                         self.resource['datacenter']['description'])
        self.assertEqual(datacenter['properties']['location'],
                         self.resource['datacenter']['location'])

    def test_get_failure(self):
        try:
            self.client.get_datacenter(datacenter_id='00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_create_failure(self):
        try:
            datacenter = Datacenter(name=self.resource['datacenter']['name'])
            self.client.create_datacenter(datacenter)
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'location',
                          e.content[0]['message'])

    def test_remove_datacenter(self):
        datacenter = self.client.create_datacenter(
            datacenter=Datacenter(**self.resource['datacenter']))
        self.client.wait_for_completion(datacenter)

        response = self.client.delete_datacenter(
            datacenter_id=datacenter['id'])

        self.assertTrue(response)
        assertRegex(self, response['requestId'], self.resource['uuid_match'])

    def test_update_datacenter(self):
        datacenter = self.client.update_datacenter(
            datacenter_id=self.datacenter['id'],
            description=self.resource['datacenter']['name']+' - RENAME')
        self.client.wait_for_completion(datacenter)
        time.sleep(10)
        datacenter = self.client.get_datacenter(datacenter_id=self.datacenter['id'])

        assertRegex(self, datacenter['id'], self.resource['uuid_match'])
        self.assertEqual(datacenter['id'], self.datacenter['id'])
        self.assertEqual(datacenter['properties']['name'], self.resource['datacenter']['name'])
        self.assertEqual(datacenter['properties']['description'],
                         self.resource['datacenter']['name']+' - RENAME')
        self.assertEqual(datacenter['properties']['location'],
                         self.resource['datacenter']['location'])
        self.assertGreater(datacenter['properties']['version'], 1)

    def test_create_simple(self):
        datacenter = self.client.create_datacenter(
            datacenter=Datacenter(**self.resource['datacenter']))
        self.client.wait_for_completion(datacenter)

        self.assertEqual(datacenter['type'], 'datacenter')
        self.assertEqual(datacenter['properties']['name'], self.resource['datacenter']['name'])
        self.assertEqual(datacenter['properties']['description'],
                         self.resource['datacenter']['description'])
        self.assertEqual(datacenter['properties']['location'],
                         self.resource['datacenter']['location'])

        response = self.client.delete_datacenter(
            datacenter_id=datacenter['id'])
        self.assertTrue(response)

    def test_create_composite(self):
        datacenter_resource = Datacenter(**self.resource['datacenter_composite'])
        datacenter_resource.servers = [Server(**self.resource['server'])]
        datacenter_resource.volumes = [Volume(**self.resource['volume'])]

        datacenter = self.client.create_datacenter(
            datacenter=datacenter_resource)
        self.client.wait_for_completion(datacenter)

        self.assertEqual(datacenter['type'], 'datacenter')
        self.assertEqual(datacenter['properties']['name'],
                         self.resource['datacenter_composite']['name'])
        self.assertEqual(datacenter['properties']['description'],
                         self.resource['datacenter_composite']['description'])
        self.assertEqual(datacenter['properties']['location'],
                         self.resource['datacenter_composite']['location'])
        self.assertGreater(len(datacenter['entities']['servers']), 0)
        self.assertGreater(len(datacenter['entities']['volumes']), 0)

        response = self.client.delete_datacenter(
            datacenter_id=datacenter['id'])
        self.assertTrue(response)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# encoding: utf-8

# Copyright 2016-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=broad-except,global-variable-undefined

'''
pb_importVM -- import a 3rd party VM

pb_importVM is a tool to create a VM that comes from another
virtualization solution.

It reads out the meta data accompanying the disk files and creates
a VM out of it. The disk images must be uploaded already.

@author:     Jürgen Buchhammer

@copyright:  2016 IONOS. All rights reserved.

@license:    Apache License 2.0

@deffield    updated: Updated
'''

import sys
import os
import traceback

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

from time import sleep
from base64 import b64decode, b64encode

import xml.etree.ElementTree as ET

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.client import Datacenter, Volume, Server
from ionosenterprise.client import NIC

__all__ = []
__version__ = 0.1
__date__ = '2016-09-30'
__updated__ = '2016-09-30'


class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg

    def __str__(self):
        return self.msg

    def __unicode__(self):
        return self.msg
# end class CLIError


def getLogin(filename, user, passwd):
    '''
    write user/passwd to login file or get them from file.
    This method is not Py3 safe (byte vs. str)
    '''
    if filename is None:
        return (user, passwd)
    isPy2 = sys.version_info[0] == 2
    if os.path.exists(filename):
        print("Using file {} for Login".format(filename))
        with open(filename, "r") as loginfile:
            encoded_cred = loginfile.read()
            print("encoded: {}".format(encoded_cred))
            if isPy2:
                decoded_cred = b64decode(encoded_cred)
            else:
                decoded_cred = b64decode(encoded_cred).decode('utf-8')
            login = decoded_cred.split(':', 1)
            return (login[0], login[1])
    else:
        if user is None or passwd is None:
            raise ValueError("user and password must not be None")
        print("Writing file {} for Login".format(filename))
        with open(filename, "wb") as loginfile:
            creds = user+":"+passwd
            if isPy2:
                encoded_cred = b64encode(creds)
            else:
                encoded_cred = b64encode(creds.encode('utf-8'))
            print("encoded: {}".format(encoded_cred))
            loginfile.write(encoded_cred)
        return (user, passwd)
# end getLogin()


def wait_for_request(pbclient, request_id,
                     timeout=0, initial_wait=5, scaleup=10):
    '''
    Waits for a request to finish until timeout.
    timeout==0 is interpreted as infinite wait time.
    Returns a tuple (return code, request status, message) where return code
    0  : request successful
    1  : request failed
    -1 : timeout exceeded
    The wait_period is increased every scaleup steps to adjust for long
    running requests.
    '''
    total_wait = 0
    wait_period = initial_wait
    next_scaleup = scaleup * wait_period
    wait = True
    while wait:
        request_status = pbclient.get_request(request_id, status=True)
        state = request_status['metadata']['status']
        if state == "DONE":
            return(0, state, request_status['metadata']['message'])
        if state == 'FAILED':
            return(1, state, request_status['metadata']['message'])
        print("Request '{}' is in state '{}'. Sleeping for {} seconds..."
              .format(request_id, state, wait_period))
        sleep(wait_period)
        total_wait += wait_period
        if timeout != 0 and total_wait > timeout:
            wait = False
        next_scaleup -= wait_period
        if next_scaleup == 0:
            wait_period += initial_wait
            next_scaleup = scaleup * wait_period
            print("scaling up wait_period to {}, next change in {} seconds"
                  .format(wait_period, next_scaleup))
    # end while(wait)
    return(-1, state, "request not finished before timeout")
# end wait_for_request()


def wait_for_requests(pbclient, request_ids=None,
                      timeout=0, initial_wait=5, scaleup=10):
    '''
    Waits for a list of requests to finish until timeout.
    timeout==0 is interpreted as infinite wait time.
    Returns a dict of request_id -> result.
    result is a tuple (return code, request status, message) where return code
    0  : request successful
    1  : request failed
    -1 : timeout exceeded
    The wait_period is increased every scaleup steps to adjust for long
    running requests.
    '''
    done = dict()
    if not request_ids:
        print("empty request list")
        return done
    total_wait = 0
    wait_period = initial_wait
    next_scaleup = scaleup * wait_period
    wait = True
    while wait:
        for request_id in request_ids:
            if request_id in done:
                continue
            request_status = pbclient.get_request(request_id, status=True)
            state = request_status['metadata']['status']
            if state == "DONE":
                done[request_id] = (0, state, request_status['metadata']['message'])
                print("Request '{}' is in state '{}'.".format(request_id, state))
            if state == 'FAILED':
                done[request_id] = (1, state, request_status['metadata']['message'])
                print("Request '{}' is in state '{}'.".format(request_id, state))
        # end for(request_ids)
        if len(done) == len(request_ids):
            wait = False
        else:
            print("{} of {} requests are finished. Sleeping for {} seconds..."
                  .format(len(done), len(request_ids), wait_period))
            sleep(wait_period)
            total_wait += wait_period
            if timeout != 0 and total_wait > timeout:
                wait = False
            next_scaleup -= wait_period
            if next_scaleup == 0:
                wait_period += initial_wait
                next_scaleup = scaleup * wait_period
                print("scaling up wait_period to {}, next change in {} seconds"
                      .format(wait_period, next_scaleup))
        # end if/else(done)
    # end while(wait)
    if len(done) != len(request_ids):
        for request_id in request_ids:
            if request_id in done:
                continue
            done[request_id] = (-1, state, "request not finished before timeout")
    return done
# end wait_for_requests()


def get_disk_image_by_name(pbclient, location, image_name):
    """
    Returns all disk images within a location with a given image name.
    The name must match exactly.
    The list may be empty.
    """
    all_images = pbclient.list_images()
    matching = [i for i in all_images['items'] if
                i['properties']['name'] == image_name and
                i['properties']['imageType'] == "HDD" and
                i['properties']['location'] == location]
    return matching
# end get_disk_image_by_name()


# -- OVF parsing --

class OFVData():
    '''OFV meta data - the data is only available after calling OFVData.parse'''
    def __init__(self, file=None):
        '''OVF meta data initializer'''
        # namespaces, dflt is default namespace xmlns, others are xmlns:<ns>
        # @TODO: read these from <Envelope> because version may change
        #        e.g. OVF 2.0 exists: xmlns:ovf="http://schemas.dmtf.org/ovf/envelope/2"
        self._ns = {
            # 'dflt': "http://schemas.dmtf.org/ovf/envelope/1", # same as ovf!?!
            'cim': "http://schemas.dmtf.org/wbem/wscim/1/common",
            'ovf': "http://schemas.dmtf.org/ovf/envelope/1",
            'rasd': "http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/"
                    "CIM_ResourceAllocationSettingData",
            'vmw': "http://www.vmware.com/schema/ovf",
            'vssd': "http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/"
                    "CIM_VirtualSystemSettingData"
        }
        self.file = file
        self.root = None
        self.name = None
        self.osid = None
        self.licenseType = "OTHER"
        self.cpus = None
        self.ram = None
        self.disks = []
        self.lans = dict()
        self.nics = []
        self.resourceTypes = {
            '1': 'Other',
            '2': 'Computer System',
            '3': 'Processor',
            '4': 'Memory',
            '5': 'IDE Controller',
            '6': 'Parallel SCSI HBA',
            '7': 'FC HBA',
            '8': 'iSCSI HBA',
            '9': 'IB HCA',
            '10': 'Ethernet Adapter',
            '11': 'Other Network Adapter',
            '12': 'I/O Slot',
            '13': 'I/O Device',
            '14': 'Floppy Drive',
            '15': 'CD Drive',
            '16': 'DVD drive',
            '17': 'Disk Drive',
            '18': 'Tape Drive',
            '19': 'Storage Extent',
            '20': 'Other storage device',
            '21': 'Serial port',
            '22': 'Parallel port',
            '23': 'USB Controller',
            '24': 'Graphics controller',
            '25': 'IEEE 1394 Controller',
            '26': 'Partitionable Unit',
            '27': 'Base Partitionable Unit',
            '28': 'Power',
            '29': 'Cooling Capacity',
            '30': 'Ethernet Switch Port',
            '31': 'Logical Disk',
            '32': 'Storage Volume',
            '33': 'Ethernet Connection',
            '..': 'DMTF reserved',
            '0x8000..0xFFFF': 'Vendor Reserved'
        }   # end resourceType
        self.osTypeOther = {
            '0': 'Unknown',
            '1': 'Other',
            '2': 'MACOS',
            '3': 'ATTUNIX',
            '4': 'DGUX',
            '5': 'DECNT',
            '6': 'Tru64 UNIX',
            '7': 'OpenVMS',
            '8': 'HPUX',
            '9': 'AIX',
            '10': 'MVS',
            '11': 'OS400',
            '12': 'OS/2',
            '13': 'JavaVM',
            '14': 'MSDOS',
            '15': 'WIN3x',
            '16': 'WIN95',
            '17': 'WIN98',
            '18': 'WINNT',
            '19': 'WINCE',
            '20': 'NCR3000',
            '21': 'NetWare',
            '22': 'OSF',
            '23': 'DC/OS',
            '24': 'Reliant UNIX',
            '25': 'SCO UnixWare',
            '26': 'SCO OpenServer',
            '27': 'Sequent',
            '28': 'IRIX',
            '29': 'Solaris',
            '30': 'SunOS',
            '31': 'U6000',
            '32': 'ASERIES',
            '33': 'HP NonStop OS',
            '34': 'HP NonStop OSS',
            '35': 'BS2000',
            '37': 'Lynx',
            '38': 'XENIX',
            '39': 'VM',
            '40': 'Interactive UNIX',
            '41': 'BSDUNIX',
            '42': 'FreeBSD',
            '43': 'NetBSD',
            '44': 'GNU Hurd',
            '45': 'OS9',
            '46': 'MACH Kernel',
            '47': 'Inferno',
            '48': 'QNX',
            '49': 'EPOC',
            '50': 'IxWorks',
            '51': 'VxWorks',
            '52': 'MiNT',
            '53': 'BeOS',
            '54': 'HP MPE',
            '55': 'NextStep',
            '56': 'PalmPilot',
            '57': 'Rhapsody',
            '59': 'Dedicated',
            '60': 'OS/390',
            '61': 'VSE',
            '62': 'TPF',
            '64': 'Caldera Open UNIX',
            '65': 'OpenBSD',
            '66': 'Not Applicable',
            '68': 'z/OS',
            '78': 'FreeBSD 64-Bit',
            '81': 'Solaris 64-Bit',
            '86': 'Novell OES',
            '87': 'Novell Linux Desktop',
            '88': 'Sun Java Desktop System',
            '102': 'Other 64-Bit',
            '104': 'VMware ESXi',
            '110': 'eComStation 32-bitx',
        }   # end osTypeOther
        self.osTypeLinux = {
            '36': 'LINUX',
            '79': 'RedHat Enterprise Linux',
            '80': 'RedHat Enterprise Linux 64-Bit',
            '82': 'SUSE',
            '83': 'SUSE 64-Bit',
            '84': 'SLES',
            '85': 'SLES 64-Bit',
            '89': 'Mandriva',
            '90': 'Mandriva 64-Bit',
            '91': 'TurboLinux',
            '92': 'TurboLinux 64-Bit',
            '93': 'Ubuntu',
            '94': 'Ubuntu 64-Bit',
            '95': 'Debian',
            '96': 'Debian 64-Bit',
            '97': 'Linux 2.4.x',
            '98': 'Linux 2.4.x 64-Bit',
            '99': 'Linux 2.6.x',
            '100': 'Linux 2.6.x 64-Bit',
            '101': 'Linux 64-Bit',
            '106': 'CentOS 32-bit',
            '107': 'CentOS 64-bit',
            '108': 'Oracle Linux 32-bit',
            '109': 'Oracle Linux 64-bit',
        }   # end osTypeLinux
        self.osTypeWindows = {
            '58': 'Windows 2000',
            '63': 'Windows (R) Me',
            '67': 'Windows XP',
            '69': 'Microsoft Windows Server 2003',
            '70': 'Microsoft Windows Server 2003 64-Bit',
            '71': 'Windows XP 64-Bit',
            '72': 'Windows XP Embedded',
            '73': 'Windows Vista',
            '74': 'Windows Vista 64-Bit',
            '75': 'Windows Embedded for Point of Service',
            '76': 'Microsoft Windows Server 2008',
            '77': 'Microsoft Windows Server 2008 64-Bit',
            '103': 'Microsoft Windows Server 2008 R2',
            '105': 'Microsoft Windows 7',
            '111': 'Microsoft Windows Server 2011',
            '113': 'Microsoft Windows Server 2012',
            '114': 'Microsoft Windows 8',
            '115': 'Microsoft Windows 8 64-bit',
            '116': 'Microsoft Windows Server 2012 R2'
        }   # end osTypeWindows
    # end __init__()

    def parse(self):
        tree = ET.parse(self.file)
        self.root = tree.getroot()
        print("parsed file, root element is '{} w/ attributes {}"
              .format(self.root.tag, self.root.attrib))
        self._collect_system_data()
        self._collect_disk_data()
        self._collect_nic_data()
    # end parse()

    def _nsattr(self, attr, ns=None):
        ''' returns an attribute name w/ namespace prefix'''
        if ns is None:
            return attr
        return '{' + self._ns[ns] + '}' + attr
    # end _nsattr()

    def _collect_system_data(self):
        virtsys = self.root.find('ovf:VirtualSystem', self._ns)
        self.name = virtsys.find('ovf:Name', self._ns).text
        virtos = virtsys.find('ovf:OperatingSystemSection', self._ns)
        self.osid = virtos.get(self._nsattr('id', 'ovf'))
        if self.osid in self.osTypeLinux:
            self.licenseType = "LINUX"
            osname = self.osTypeLinux[self.osid]
        else:
            if self.osid in self.osTypeWindows:
                self.licenseType = "WINDOWS"
                osname = self.osTypeWindows[self.osid]
            else:
                osname = self.osTypeOther[self.osid]
        print("VM '{}' has {}-type OS '{}'(id:{})"
              .format(self.name, self.licenseType, osname, self.osid))
        virtcpu = virtsys.find('./ovf:VirtualHardwareSection/ovf:Item/[rasd:ResourceType="3"]',
                               self._ns)
        self.cpus = virtcpu.find('rasd:VirtualQuantity', self._ns).text
        # !!! VMware also as vmw:CoresPerSocket !!!
        # we currently exclude this, so there may be cores missing in VM!
        virtmem = virtsys.find('./ovf:VirtualHardwareSection/ovf:Item/[rasd:ResourceType="4"]',
                               self._ns)
        # we assume that RAM is specified in MB (or 'byte * 2^20')
        self.ram = virtmem.find('rasd:VirtualQuantity', self._ns).text
        print("VM '{}' has {} CPUs and {} MB RAM"
              .format(self.name, self.cpus, self.ram))
    # end _collect_system_data()

    # get the disks
    def _collect_disk_data(self):
        filerefs = self.root.findall('./ovf:References/ovf:File', self._ns)
        files = dict()
        for ref in filerefs:
            name = ref.get(self._nsattr('href', 'ovf'))
            fileid = ref.get(self._nsattr('id', 'ovf'))
            files[fileid] = name
        print("found filerefs {}".format(files))
        diskrefs = self.root.findall('./ovf:DiskSection/ovf:Disk', self._ns)
        disks = dict()
        for ref in diskrefs:
            # Note: we assume ovf:capacityAllocationUnits="byte * 2^30" == GiB
            capacity = ref.get(self._nsattr('capacity', 'ovf'))
            # reference to file references above
            fref = ref.get(self._nsattr('fileRef', 'ovf'))
            # the virt. HW section refers to '/disk/vmdisk1' not 'vmdisk1'
            diskid = 'ovf:/disk/'+ref.get(self._nsattr('diskId', 'ovf'))
            # we resolve fref here, we only need the name from now on
            disks[diskid] = {'capacity': capacity, 'file': files[fref]}
        print("found disks {}".format(disks))
        virtsys = self.root.find('ovf:VirtualSystem', self._ns)
        virthds = virtsys.findall('./ovf:VirtualHardwareSection/ovf:Item/[rasd:ResourceType="17"]',
                                  self._ns)
        devices = dict()
        for hdd in virthds:
            # print("hdd is {}".format(hdd))
            diskref = hdd.find('rasd:HostResource', self._ns).text
            address = hdd.find('rasd:AddressOnParent', self._ns)
            if address is None:
                print("no address for sorting found for {}, use InstanceId"
                      .format(diskref))
                devNr = hdd.find('rasd:InstanceId', self._ns).text
                print("disk {} has InstanceId {}".format(diskref, devNr))
            else:
                devNr = address.text
                print("disk {} has address {}".format(diskref, devNr))
            devices[devNr] = disks[diskref]
        print("devices : {}".format(devices))
        self.disks = [devices[dev_no] for dev_no in sorted(devices)]
        print("disks : {}".format(self.disks))
    # end _collect_disk_data()

    # get the nics
    def _collect_nic_data(self):
        # the NetworkSection contains only name and description
        # maybe that helps for better LAN assignment one day
        vnets = self.root.findall('./ovf:NetworkSection/ovf:Network', self._ns)
        lanid = 1
        for net in vnets:
            self.lans[net.get(self._nsattr('name', 'ovf'))] = lanid
            lanid += 1
        print("LANs found: {}".format(self.lans))
        virtsys = self.root.find('ovf:VirtualSystem', self._ns)
        virtnics = virtsys.findall(
            './ovf:VirtualHardwareSection/ovf:Item/[rasd:ResourceType="10"]', self._ns)
        devices = dict()
        for nic in virtnics:
            # print("nic is {}".format(nic))
            nicname = nic.find('rasd:ElementName', self._ns).text
            connection = nic.find('rasd:Connection', self._ns).text
            address = nic.find('rasd:AddressOnParent', self._ns)
            if address is None:
                print("no address for sorting found for {}, use InstanceId"
                      .format(nicname))
                devNr = nic.find('rasd:InstanceId', self._ns).text
                print("nic '{}' has InstanceId {}".format(nicname, devNr))
            else:
                devNr = address.text
                print("nic '{}' has address {}".format(nicname, devNr))
            devices[devNr] = {'nic': nicname, 'lan': connection,
                              'lanid': self.lans[connection]}
        print("devices : {}".format(devices))
        self.nics = [devices[dev_no] for dev_no in sorted(devices)]
        print("nics : {}".format(self.nics))
    # end _collect_nic_data()
# end class OVFData


# -- MAIN --

def main(argv=None):
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version,
                                                     program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by Jürgen Buchhammer on %s.
  Copyright 2016 IONOS. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license,
                                formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-u', '--user', dest='user', help='the login name')
        parser.add_argument('-p', '--password', dest='password',
                            help='the login password')
        parser.add_argument('-L', '--Login', dest='loginfile', default=None,
                            help='the login file to use')
        parser.add_argument('-t', '--type', dest='metatype',
                            default="OVF",
                            help='type of VM meta data')
        parser.add_argument('-m', '--metadata', dest='metafile',
                            required=True, default=None,
                            help='meta data file')
        parser.add_argument('-d', '--datacenterid', dest='datacenterid',
                            default=None,
                            help='datacenter of the new server')
        parser.add_argument('-D', '--DCname', dest='dcname', default=None,
                            help='new datacenter name')
        parser.add_argument('-l', '--location', dest='location', default=None,
                            help='location for new datacenter')
        parser.add_argument('-v', '--verbose', dest="verbose", action="count",
                            default=0,
                            help="set verbosity level [default: %(default)s]")
        parser.add_argument('-V', '--version', action='version',
                            version=program_version_message)

        # Process arguments
        args = parser.parse_args()
        global verbose
        verbose = args.verbose

        if verbose > 0:
            print("Verbose mode on")
            print("start {} with args {}".format(program_name, str(args)))

        (user, password) = getLogin(args.loginfile, args.user, args.password)
        if user is None or password is None:
            raise ValueError("user or password resolved to None")
        pbclient = IonosEnterpriseService(user, password)

        if args.metatype == 'OVF':
            metadata = OFVData(args.metafile)
            metadata.parse()
        else:
            sys.stderr.write("Metadata type '{}' is not supported"
                             .format(args.metatype))
            return 1

        # we need the DC first to have the location defined
        dc_id = None
        if args.datacenterid is None:
            if args.dcname is None or args.location is None:
                sys.stderr.write("Either '-d <id>' or '-D <name> -l <loc>'  must be specified")
                return 1
            # else: we will create the DC later after parsing the meta data
        else:
            dc_id = args.datacenterid

        if dc_id is None:
            location = args.location
            dc = Datacenter(name=args.dcname, location=location,
                            description="created by pb_importVM")
            print("create new DC {}".format(str(dc)))
            response = pbclient.create_datacenter(dc)
            dc_id = response['id']
            result = wait_for_request(pbclient, response['requestId'])
            print("wait loop returned {}".format(result))
        else:
            dc = pbclient.get_datacenter(dc_id)
            location = dc['properties']['location']
            print("use existing DC {} in location {}"
                  .format(dc['properties']['name'], location))

        # check if images exist
        for disk in metadata.disks:
            disk_name = disk['file']
            images = get_disk_image_by_name(pbclient, location, disk_name)
            if not images:
                raise ValueError("No HDD image with name '{}' found in location {}"
                                 .format(disk_name, location))
            if len(images) > 1:
                raise ValueError("Ambigous image name '{}' in location {}"
                                 .format(disk_name, location))
            disk['image'] = images[0]['id']

        # now we're ready to create the VM
        # Server
        server = Server(name=metadata.name,
                        cores=metadata.cpus, ram=metadata.ram)
        print("create server {}".format(str(Server)))
        response = pbclient.create_server(dc_id, server)
        srv_id = response['id']
        result = wait_for_request(pbclient, response['requestId'])
        print("wait loop returned {}".format(str(result)))
        # NICs (note that createing LANs may be implicit)
        for nic in metadata.nics:
            dcnic = NIC(name=nic['nic'], lan=nic['lanid'])
            print("create NIC {}".format(str(dcnic)))
            response = pbclient.create_nic(dc_id, srv_id, dcnic)
            nic_id = response['id']
            result = wait_for_request(pbclient, response['requestId'])
            print("wait loop returned {}".format(str(result)))
            response = pbclient.get_nic(dc_id, srv_id, nic_id, 2)
            mac = response['properties']['mac']
            print("dcnic has MAC {} for {}".format(mac, nic_id))
        # end for(nics)
        # Volumes (we use the image name as volume name too
        requests = []
        for disk in metadata.disks:
            dcvol = Volume(name=disk['file'], size=disk['capacity'],
                           image=disk['image'],
                           licence_type=metadata.licenseType)
            print("create Volume {}".format(str(dcvol)))
            response = pbclient.create_volume(dc_id, dcvol)
            requests.append(response['requestId'])
            disk['volume_id'] = response['id']
        # end for(disks)
        if requests:
            result = wait_for_requests(pbclient, requests, initial_wait=10, scaleup=15)
            print("wait loop returned {}".format(str(result)))
        for disk in metadata.disks:
            print("attach volume {}".format(disk))
            response = pbclient.attach_volume(dc_id, srv_id, disk['volume_id'])
            result = wait_for_request(pbclient, response['requestId'])
            print("wait loop returned {}".format(str(result)))
        # end for(disks)

        print("import of VM succesfully finished")
        return 0

    except KeyboardInterrupt:
        # handle keyboard interrupt
        return 0
    except Exception:
        traceback.print_exc()
        sys.stderr.write("\n" + program_name + ":  for help use --help\n")
        return 2
# end main()


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ICError(Exception):
    """Base error for this module."""
    def __init__(self, resp, content, uri=None):  # pylint: disable=super-init-not-called
        self.resp = resp
        self.content = content
        self.uri = uri


class ICNotAuthorizedError(ICError):
    """The authorization information provided is not correct"""


class ICNotFoundError(ICError):
    """The IonosEnterprise entity was not found"""


class ICValidationError(ICError):
    """The HTTP data provided is not valid"""


class ICRateLimitExceededError(ICError):
    """The number of requests sent have exceeded the allowed API rate limit"""


class ICRequestError(Exception):
    """Base error for request failures"""
    def __init__(self, msg, request_id):  # pylint: disable=super-init-not-called
        self.msg = msg
        self.request_id = request_id


class ICFailedRequest(ICRequestError):
    """Raised when a provisioning request failed."""


class ICTimeoutError(ICRequestError):
    """Raised when a request does not finish in the given time span."""



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IonosEnterpriseService
from ionosenterprise.items import Datacenter


class TestK8sNodepools(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        # Create test k8s cluster
        cls.datacenter = cls.client.create_datacenter(
            datacenter=Datacenter(**cls.resource['datacenter']))
        cls.k8s_cluster = cls.client.create_k8s_cluster(**cls.resource['k8s_cluster'])

        # Wait for k8s cluster to be active
        cls.client.wait_for(
            fn_request=lambda: cls.client.list_k8s_clusters(),
            fn_check=lambda r: list(filter(
                lambda e: e['id'] == cls.k8s_cluster['id'],
                r['items']
            ))[0]['metadata']['state'] == 'ACTIVE',
            scaleup=10000
        )

        cls.k8s_cluster_nodepool1 = cls.client.create_k8s_cluster_nodepool(
            cls.k8s_cluster['id'],
            datacenter_id=cls.datacenter['id'],
            **cls.resource['k8s_nodepool']
        )

        cls.k8s_cluster_nodepool2 = cls.client.create_k8s_cluster_nodepool(
            cls.k8s_cluster['id'],
            datacenter_id=cls.datacenter['id'],
            **cls.resource['k8s_nodepool']
        )

        # Wait for k8s cluster nodepool 1 to be active
        cls.client.wait_for(
            fn_request=lambda: cls.client.get_k8s_cluster_nodepool(
                cls.k8s_cluster['id'],
                cls.k8s_cluster_nodepool1['id']),
            fn_check=lambda r: r['metadata']['state'] == 'ACTIVE',
            scaleup=10000
        )

        # Wait for k8s cluster nodepool 2 to be active
        cls.client.wait_for(
            fn_request=lambda: cls.client.get_k8s_cluster_nodepool(
                cls.k8s_cluster['id'],
                cls.k8s_cluster_nodepool2['id']),
            fn_check=lambda r: r['metadata']['state'] == 'ACTIVE',
            scaleup=10000
        )

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_k8s_cluster_nodepool(cls.k8s_cluster['id'],
                                               cls.k8s_cluster_nodepool1['id'])
        # Wait for k8s cluster nodepool to be deleted
        cls.client.wait_for(
            fn_request=lambda: cls.client.self.client.list_k8s_cluster_nodepools(cls.k8s_cluster['id']),
            fn_check=lambda r: len(list(filter(
                lambda e: e['id'] == cls.k8s_cluster_nodepool1['id'],
                r['items']
            ))) == 0,
            scaleup=10000
        )
        cls.client.delete_k8s_cluster(cls.k8s_cluster['id'])
        cls.client.delete_datacenter(cls.datacenter['id'])

    def test_list_k8s_nodepools(self):
        k8s_nodepools = self.client.list_k8s_cluster_nodepools(self.k8s_cluster['id'])
        self.assertGreater(len(k8s_nodepools), 0)
        self.assertGreater(len(k8s_nodepools['items']), 0)

    def test_get_k8s_nodepool(self):
        k8s_nodepool = self.client.get_k8s_cluster_nodepool(self.k8s_cluster['id'],
                                                            self.k8s_cluster_nodepool1['id'])
        self.assertEqual(k8s_nodepool['type'], 'nodepool')
        self.assertEqual(k8s_nodepool['id'], self.k8s_cluster_nodepool1['id'])
        self.assertEqual(k8s_nodepool['properties']['name'], self.k8s_cluster_nodepool1['properties']['name'])

    def test_delete_k8s_nodepool(self):
        response = self.client.delete_k8s_cluster_nodepool(self.k8s_cluster['id'],
                                                           self.k8s_cluster_nodepool2['id'])
        print(response)
        self.assertTrue('requestId' in response)

    def test_update_k8s_nodepool(self):
        response = self.client.update_k8s_cluster_nodepool(
            self.k8s_cluster['id'], self.k8s_cluster_nodepool1['id'], 2,
            maintenance_window={
                'dayOfTheWeek': "Monday",
                'time': '17:00:00'},
            auto_scaling={'minNodeCount': '2', 'maxNodeCount': '3'}
        )
        # Wait for k8s cluster nodepool 1 to be active
        self.client.wait_for(
            fn_request=lambda: self.client.get_k8s_cluster_nodepool(
                self.k8s_cluster['id'],
                self.k8s_cluster_nodepool1['id']),
            fn_check=lambda r: r['metadata']['state'] == 'ACTIVE',
            scaleup=10000
        )
        self.assertEqual(response['maintenance_window']['dayOfTheWeek'], 'Monday')
        self.assertEqual(response['node_count'], 2)
        self.assertEqual(response['auto_scaling']['maxNodeCount'], 3)



#!/usr/bin/python3

# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=pointless-string-statement,reimported,wrong-import-position

"""List Images
"""
from ionosenterprise.client import IonosEnterpriseService

client = IonosEnterpriseService(
    username='username', password='password')

images = client.list_images()

print(images)

"""
Update Image

Valid image parameters are:

* name (str)
* description (str)
* licence_type (one of 'LINUX', 'WINDOWS' or 'UNKNOWN')
* cpu_hot_plug (bool)
* ram_hot_plug (bool)
* nic_hot_plug (bool)
* nic_hot_unplug (bool)
* disc_virtio_hot_plug (bool)
* disc_virtio_hot_unplug (bool)
* disc_scsi_hot_plug (bool)
* disc_scsi_hot_unplug (bool)

"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

client = IonosEnterpriseService(
    username='username', password='password')

image_id = '7df81087-5835-41c6-a10b-3e098593bbd2'
image = client.update_image(
    image_id,
    name='New name',
    description="Centos 7 with NGnix",
    licence_type='LINUX',
    cpu_hot_plug=True,
    ram_hot_plug=True,
    nic_hot_plug=True,
    nic_hot_unplug=True,
    disc_virtio_hot_plug=True,
    disc_virtio_hot_unplug=True)

"""Delete Image
"""
from ionosenterprise.client import IonosEnterpriseService  # noqa

client = IonosEnterpriseService(
    username='username', password='password')

image_id = '7df81087-5835-41c6-a10b-3e098593bbd2'

image = client.delete_image(image_id)



# Copyright 2015-2017 IONOS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import warnings
import unittest

from six import assertRegex

from helpers import configuration
from helpers.resources import resource

from ionosenterprise.client import IPBlock, IonosEnterpriseService
from ionosenterprise.errors import ICError, ICNotFoundError


class TestIPBlock(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        warnings.filterwarnings("ignore", category=ResourceWarning,
                                message="unclosed.*<ssl.SSLSocket.*>")
        cls.resource = resource()
        cls.client = IonosEnterpriseService(
            username=configuration.USERNAME,
            password=configuration.PASSWORD,
            headers=configuration.HEADERS)

        ipblock1 = IPBlock(**cls.resource['ipblock'])
        ipblock1.size = 2
        cls.ipblock1 = cls.client.reserve_ipblock(ipblock1)

        ipblock2 = IPBlock(**cls.resource['ipblock'])
        cls.ipblock2 = cls.client.reserve_ipblock(ipblock2)

    @classmethod
    def tearDownClass(cls):
        cls.client.delete_ipblock(cls.ipblock1['id'])

    def test_list_ipblocks(self):
        ipblocks = self.client.list_ipblocks()

        assertRegex(self, ipblocks['items'][0]['id'], self.resource['uuid_match'])
        self.assertGreater(len(ipblocks['items']), 0)
        self.assertEqual(ipblocks['items'][0]['type'], 'ipblock')
        self.assertGreater(ipblocks['items'][0]['properties']['size'], 0)
        self.assertIn(ipblocks['items'][0]['properties']['location'], self.resource['locations'])

    def test_get_ipblock(self):
        ipblock = self.client.get_ipblock(self.ipblock1['id'])

        assertRegex(self, ipblock['id'], self.resource['uuid_match'])
        self.assertEqual(ipblock['id'], self.ipblock1['id'])
        self.assertEqual(ipblock['type'], 'ipblock')
        self.assertEqual(ipblock['properties']['name'], (self.resource['ipblock']['name']))
        self.assertEqual(ipblock['properties']['size'], 2)
        self.assertEqual(len(ipblock['properties']['ips']), 2)
        self.assertEqual(ipblock['properties']['location'], self.resource['ipblock']['location'])

    def test_delete_ipblock(self):
        ipblock = self.client.delete_ipblock(self.ipblock2['id'])

        self.assertTrue(ipblock)
        assertRegex(self, ipblock['requestId'], self.resource['uuid_match'])

    def test_reserve_ipblock(self):
        ipblock = self.client.reserve_ipblock(IPBlock(**self.resource['ipblock']))

        assertRegex(self, ipblock['id'], self.resource['uuid_match'])
        self.assertEqual(ipblock['properties']['name'], (self.resource['ipblock']['name']))
        self.assertEqual(ipblock['properties']['size'], self.resource['ipblock']['size'])
        self.assertEqual(ipblock['properties']['location'], self.resource['ipblock']['location'])

        self.client.delete_ipblock(ipblock['id'])

    def test_get_failure(self):
        try:
            self.client.get_ipblock('00000000-0000-0000-0000-000000000000')
        except ICNotFoundError as e:
            self.assertIn(self.resource['not_found_error'], e.content[0]['message'])

    def test_reserve_failure(self):
        try:
            ipblock = IPBlock(name=self.resource['ipblock']['name'], size=1)
            self.client.reserve_ipblock(ipblock)
        except ICError as e:
            self.assertIn(self.resource['missing_attribute_error'] % 'location',
                          e.content[0]['message'])


if __name__ == '__main__':
    unittest.main()
